#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.search.portlet.interfaces module

"""

from zope.interface import Attribute, Interface
from zope.schema import Bool

from pyams_i18n.schema import I18nTextLineField
from pyams_portal.interfaces import IPortletSettings

__docformat__ = 'restructuredtext'

from pyams_content import _


SEARCH_RESULTS_PORTLET_NAME = 'pyams_content.portlet.search.results'
SEARCH_RESULTS_ICON_CLASS = 'fas fa-search'

SEARCH_RESULTS_PORTLET_FLAG = 'pyams_content.portlet.search.has_results'
"""Request annotations flag for portlet search results"""


class ISearchResultsAggregates(Interface):
    """Aggregated search results portlet renderer marker interface"""

    aggregates = Attribute("Search results aggregates")


class ISearchResultsPortletSettings(IPortletSettings):
    """Search results portlet settings"""

    title = I18nTextLineField(title=_("Title"),
                              description=_("Portlet main title"),
                              required=False)

    allow_empty_query = Bool(title=_("Allow empty query?"),
                             description=_("If 'no', no result will be displayed if user didn't "
                                           "entered a search string"),
                             required=True,
                             default=True)

    force_canonical_url = Bool(title=_("Force canonical URL?"),
                               description=_("By default, internal links use a \"relative\" URL, "
                                             "which tries to display link target in the current "
                                             "context; by using a canonical URL, you can display "
                                             "target in it's attachment context (if defined)"),
                               required=False,
                               default=False)

    def has_user_query(self):
        """Check if user entered custom search arguments"""

    def get_items(self, request=None, limit=None, ignore_cache=False):
        """Get search results"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.search.interfaces module

"""

from zope.interface import Attribute, Interface, Invalid, invariant
from zope.schema import Bool, Choice, Set

from pyams_content.interfaces import IBaseContent
from pyams_content.shared.common import VIEWS_SHARED_CONTENT_TYPES_VOCABULARY
from pyams_content.shared.common.interfaces.types import ALL_DATA_TYPES_VOCABULARY
from pyams_content.shared.site.interfaces import IBaseSiteItem, ISiteElement
from pyams_content.shared.view.interfaces import IWfView, RELEVANCE_ORDER, USER_VIEW_ORDER_VOCABULARY
from pyams_content.shared.view.interfaces.query import IViewQuery
from pyams_i18n.schema import I18nTextLineField
from pyams_sequence.interfaces import IInternalReferencesList, ISequentialIdTarget
from pyams_sequence.schema import InternalReferenceField

__docformat__ = 'restructuredtext'

from pyams_content import _


class ISearchManagerInfo(IInternalReferencesList):
    """Search manager interface"""

    reference = InternalReferenceField(title=_("Main search engine"),
                                       description=_("Search folder handling main site search. "
                                                     "You can search a reference using '+' "
                                                     "followed by internal number, or by "
                                                     "entering text matching content title."),
                                       required=False)

    search_target = Attribute("Search target object")

    name = I18nTextLineField(title=_("Search engine name"),
                             description=_("Name given to the search engine"),
                             required=False)

    description = I18nTextLineField(title=_("Description"),
                                    description=_("Description given to the search engine"),
                                    required=False)

    enable_tags_search = Bool(title=_("Enable search by tag?"),
                              description=_("If 'yes', displayed tags will lead to a search "
                                            "engine displaying contents matching given tag"),
                              required=True,
                              default=False)

    tags_search_target = InternalReferenceField(title=_("Tags search target"),
                                                description=_("Site or folder where tags search "
                                                              "is displayed"),
                                                required=False)

    tags_target = Attribute("Tags search target object reference")

    @invariant
    def check_tags_search_target(self):
        """Check that target is defined to enable tags search"""
        if self.enable_tags_search and not self.tags_search_target:
            raise Invalid(_("You must specify search target when activating search by tags!"))

    enable_collections_search = Bool(title=_("Enable search by collection?"),
                                     description=_("If 'yes', displayed collections will lead to "
                                                   "a search engine displaying contents matching "
                                                   "given collection"),
                                     required=True,
                                     default=False)

    collections_search_target = InternalReferenceField(title=_("Collections search target"),
                                                       description=_("Site or folder where collections "
                                                                     "search is displayed"),
                                                       required=False)

    collections_target = Attribute("Collections search target object reference")

    @invariant
    def check_collections_search_target(self):
        """Check that target is defined to enable collections search"""
        if self.enable_collections_search and not self.collections_search_target:
            raise Invalid(_("You must specify search target when activating search by "
                            "collections!"))


class ISearchFolder(IBaseContent, IBaseSiteItem, ISiteElement, IWfView, ISequentialIdTarget):
    """Search folder interface"""

    order_by = Choice(title=_("Order by"),
                      description=_("Property to use to sort results; publication date can be "
                                    "different from first publication date for contents which "
                                    "have been retired and re-published with a different "
                                    "publication date"),
                      vocabulary=USER_VIEW_ORDER_VOCABULARY,
                      required=False,
                      default=RELEVANCE_ORDER)

    visible_in_list = Bool(title=_("Visible in folders list"),
                           description=_("If 'no', search folder will not be displayed into "
                                         "parent's contents list"),
                           required=True,
                           default=True)

    navigation_title = I18nTextLineField(title=_("Navigation title"),
                                         description=_("Folder's title displayed in navigation "
                                                       "pages; original title will be used if "
                                                       "none is specified"),
                                         required=False)

    selected_content_types = Set(title=_("Selected content types"),
                                 description=_("Searched content types; leave empty for all"),
                                 value_type=Choice(vocabulary=VIEWS_SHARED_CONTENT_TYPES_VOCABULARY),
                                 required=False)

    selected_datatypes = Set(title=_("Selected data types"),
                             description=_("Searched data types; leave empty for all"),
                             value_type=Choice(vocabulary=ALL_DATA_TYPES_VOCABULARY),
                             required=False)


class ISearchFolderQuery(IViewQuery):
    """Search folder query interface"""


class IContextUserSearchSettings(Interface):
    """Context user search settings interface

    This interface is used to get user search settings from context.
    """

    def get_settings(self):
        """Get matching context settings

        :return dict: user search settings mapping
        """


class ISearchFormRequestParams(Interface):
    """User search request params

    This interface is used to get user request params which can be included into
    search forms.
    """

    def get_params(self):
        """Get request params

        :return iterator: iterator over list of request params; each element may be a mapping
        containing param attributes 'name' and 'value'
        """



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.topic module

"""

from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.illustration.interfaces import IIllustrationTarget, \
    ILinkIllustrationTarget
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget
from pyams_content.component.thesaurus.interfaces import ITagsTarget, IThemesTarget
from pyams_content.feature.preview.interfaces import IPreviewTarget
from pyams_content.feature.review import IReviewTarget
from pyams_content.shared.common import ISharedContent, IWfSharedContent, SharedContent, \
    WfSharedContent
from pyams_content.shared.common.types import WfTypedSharedContentMixin
from pyams_content.shared.topic.interfaces import ITopic, IWfTopic, TOPIC_CONTENT_NAME, \
    TOPIC_CONTENT_TYPE
from pyams_utils.factory import factory_config


__docformat__ = 'restructuredtext'


@factory_config(IWfTopic)
@factory_config(IWfSharedContent, name=TOPIC_CONTENT_TYPE)
@implementer(IIllustrationTarget, ILinkIllustrationTarget, IParagraphContainerTarget,
             ITagsTarget, IThemesTarget, IReviewTarget, IPreviewTarget)
class WfTopic(WfSharedContent, WfTypedSharedContentMixin):
    """Base topic"""

    content_type = TOPIC_CONTENT_TYPE
    content_name = TOPIC_CONTENT_NAME
    content_intf = IWfTopic
    content_view = True

    references = FieldProperty(IWfTopic['references'])
    data_type = FieldProperty(IWfTopic['data_type'])


@factory_config(ITopic)
@factory_config(ISharedContent, name=TOPIC_CONTENT_TYPE)
class Topic(SharedContent):
    """Workflow managed topic class"""

    content_type = TOPIC_CONTENT_TYPE
    content_name = TOPIC_CONTENT_NAME



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.skin.interfaces module

"""

from pyams_content.feature.renderer import IRendererSettings
from pyams_skin.schema import BootstrapThumbnailsSelectionField


__docformat__ = 'restructuredtext'

from pyams_content import _


class IGalleryDefaultRendererSettings(IRendererSettings):
    """Gallery default renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Thumbnails selection"),
        description=_("Selection used to display images thumbnails"),
        default_width=2,
        change_width=True,
        required=False)

    def get_css_cols(self):
        """Return CSS columns width matching current selection"""


class IGalleryCarouselRendererSettings(IRendererSettings):
    """Gallery carousel renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Images selection"),
        description=_("Selection used to display images"),
        default_width=12,
        change_width=False,
        required=False)


class IGalleryRandomImageRendererSettings(IRendererSettings):
    """Gallery random image renderer settings interface"""

    thumb_selection = BootstrapThumbnailsSelectionField(
        title=_("Images selection"),
        description=_("Selection used to display images"),
        default_width=12,
        change_width=False,
        required=False)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.reference module

"""

from persistent import Persistent
from pyramid.events import subscriber
from zope.component.interfaces import ISite
from zope.container.contained import Contained
from zope.container.folder import Folder
from zope.interface import implementer
from zope.lifecycleevent import IObjectAddedEvent
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.interfaces import MANAGE_REFERENCE_TABLE_PERMISSION
from pyams_content.reference.interfaces import IReferenceInfo, IReferenceManager, IReferenceTable, IReferenceTableRoles, \
    REFERENCE_TABLE_ROLES
from pyams_i18n.interfaces import II18n, II18nManager
from pyams_security.interfaces import IDefaultProtectionPolicy, IRolesPolicy, IViewContextPermissionChecker
from pyams_security.property import RolePrincipalsFieldProperty
from pyams_security.security import ProtectedObjectMixin, ProtectedObjectRoles
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.registry import query_utility
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent

__docformat__ = 'restructuredtext'


@factory_config(IReferenceManager)
class ReferencesManager(Folder):
    """References tables container"""

    title = FieldProperty(IReferenceManager['title'])
    short_name = FieldProperty(IReferenceManager['short_name'])

    def __init__(self):
        super().__init__()
        self.title = {
            'en': 'References tables',
            'fr': 'Tables de références'
        }
        self.short_name = self.title.copy()


@subscriber(IObjectAddedEvent, context_selector=IReferenceManager)
def handle_added_references_manager(event):
    """Handle new references manager"""
    site = get_parent(event.object, ISite)
    registry = site.getSiteManager()
    if registry is not None:
        registry.registerUtility(event.object, IReferenceManager)


@implementer(IDefaultProtectionPolicy, IReferenceTable, II18nManager)
class ReferenceTable(ProtectedObjectMixin, Folder):
    """References table"""

    title = FieldProperty(IReferenceTable['title'])
    short_name = FieldProperty(IReferenceTable['short_name'])

    languages = FieldProperty(II18nManager['languages'])


@implementer(IReferenceTableRoles)
class ReferenceTableRoles(ProtectedObjectRoles):
    """References table roles"""

    managers = RolePrincipalsFieldProperty(IReferenceTableRoles['managers'])


@adapter_config(required=IReferenceTable,
                provides=IReferenceTableRoles)
def reference_table_roles(context):
    """References table roles adapter"""
    return ReferenceTableRoles(context)


@adapter_config(name=REFERENCE_TABLE_ROLES,
                required=IReferenceTable,
                provides=IRolesPolicy)
class ReferenceTableRolesPolicy(ContextAdapter):
    """References table roles policy"""

    roles_interface = IReferenceTableRoles
    weight = 10


@implementer(IReferenceInfo)
class ReferenceInfo(Persistent, Contained):
    """Reference record"""

    title = FieldProperty(IReferenceInfo['title'])
    short_name = FieldProperty(IReferenceInfo['short_name'])


@adapter_config(required=IReferenceInfo,
                provides=IViewContextPermissionChecker)
class ReferenceInfoPermissionChecker(ContextAdapter):
    """Reference info permission checker"""

    edit_permission = MANAGE_REFERENCE_TABLE_PERMISSION


class ReferencesVocabulary(SimpleVocabulary):
    """Base references vocabulary"""

    table_interface = None

    def __init__(self, context=None):
        table = query_utility(self.table_interface)
        if table is not None:
            request = check_request()
            terms = sorted([
                SimpleTerm(v.__name__,
                           title=II18n(v).query_attribute('title', request=request))
                for v in table.values()
            ], key=lambda t: t.title)
        else:
            terms = []
        super().__init__(terms)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_content.feature.filter import IFilter
from pyams_content.feature.filter.interfaces import IAggregatedPortletRendererSettings, IFilterProcessor, \
    IFilterProcessorAggregationsHandler
from pyams_layer.interfaces import IPyAMSLayer
from pyams_utils.adapter import ContextRequestAdapter, adapter_config

__docformat__ = 'restructuredtext'


@adapter_config(required=(IFilter, IPyAMSLayer, IAggregatedPortletRendererSettings),
                provides=IFilterProcessor)
class BaseFilterProcessor:
    """Base filter processor"""

    def __init__(self, filter, request, renderer_settings):
        self.filter = filter
        self.request = request
        self.renderer_settings = renderer_settings

    def process(self, aggregations, filter_type=None):
        filter = self.filter
        request = self.request
        filter_type = filter_type or filter.filter_type
        aggr = self.get_aggregations(aggregations[filter_type])
        if not aggr:
            return None
        handler = request.registry.queryMultiAdapter((filter, request, self.renderer_settings),
                                                     IFilterProcessorAggregationsHandler)
        if handler is not None:
            return handler.get_aggregations(aggr, filter_type)

    def get_aggregations(self, aggregations):
        raise NotImplementedError



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.authorization import Authenticated
from pyramid.traversal import lineage
from zope.interface import Interface

from pyams_content.feature.header.interfaces import IPageHeaderTitle
from pyams_content.feature.header.portlet import IPageHeaderPortletSettings
from pyams_content.root import ISiteRootInfos
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_portal.skin import PortletRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config

__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(IPortalContext, IPyAMSLayer, Interface, IPageHeaderPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/header-default.pt', layer=IPyAMSLayer)
class PageHeaderPortletDefaultRenderer(PortletRenderer):
    """Page header portlet default renderer"""

    label = _("Simple banner (default)")
    weight = 10

    use_authentication = True

    @property
    def logo(self):
        infos = ISiteRootInfos(self.request.root, None)
        if infos is None:
            return None, None
        return self.request.root, infos.logo

    @property
    def title(self):
        request  =self.request
        registry = request.registry
        for context in lineage(self.context):
            header = registry.queryMultiAdapter((context, self.request), IPageHeaderTitle)
            if header is not None:
                return header
        infos = ISiteRootInfos(self.request.root, None)
        return II18n(infos).query_attribute('title', request=self.request)

    @property
    def authenticated(self):
        identity = self.request.identity
        if identity is None:
            return False
        return Authenticated in identity.principals



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.navigation;portlet.interfaces module

"""

from zope.interface import Attribute, Interface

from pyams_content.feature.navigation import IMenusContainerTarget
from pyams_content.feature.navigation.interfaces import IMenuLinksContainerTarget
from pyams_i18n.schema import I18nTextLineField
from pyams_portal.interfaces import IPortletSettings


__docformat__ = 'restructuredtext'

from pyams_content import _


class ISimpleNavigationPortletSettings(IMenuLinksContainerTarget, IPortletSettings):
    """Simple navigation portlet settings interface"""

    title = I18nTextLineField(title=_("Title"),
                              description=_("Portlet main title"),
                              required=False)

    links = Attribute("Navigation links")


class ISimpleNavigationMenu(Interface):
    """Simple navigation menu marker interface"""


class IDoubleNavigationPortletSettings(IMenusContainerTarget, IPortletSettings):
    """Double navigation portlet settings interface"""

    title = I18nTextLineField(title=_("Title"),
                              description=_("Portlet main title"),
                              required=False)

    menus = Attribute("Navigation menus")



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.seo.zmi module

MThis module provides management interface components for SEO related information.
"""

from pyams_content.feature.seo import ISEOContentInfo
from pyams_content.shared.common import IWfSharedContent
from pyams_form.field import Fields
from pyams_form.interfaces.form import IGroup
from pyams_utils.adapter import adapter_config
from pyams_zmi.form import FormGroupSwitcher
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.form import IPropertiesEditForm

__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(name='seo-config.group',
                context=(IWfSharedContent, IAdminLayer, IPropertiesEditForm),
                provides=IGroup)
class SEOConfigurationEditGroup(FormGroupSwitcher):
    """SEO configuration edit form"""

    legend = _("SEO configuration")
    fields = Fields(ISEOContentInfo)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.extfile module

This module provides components which are used to handle external files.
"""

import os

from pyramid.events import subscriber
from zope.interface import alsoProvides, implementer
from zope.lifecycleevent import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent, \
    ObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.association import AssociationItem
from pyams_content.component.association.interfaces import IAssociationInfo
from pyams_content.component.extfile.interfaces import EXTAUDIO_ICON_CLASS, EXTAUDIO_ICON_HINT, \
    EXTFILE_ICON_CLASS, EXTFILE_ICON_HINT, EXTIMAGE_ICON_CLASS, EXTIMAGE_ICON_HINT, \
    EXTVIDEO_ICON_CLASS, \
    EXTVIDEO_ICON_HINT, \
    IBaseExtFile, \
    IExtAudio, IExtFile, \
    IExtFileManagerInfo, IExtImage, IExtVideo
from pyams_content.shared.common import IWfSharedContent
from pyams_file.file import EXTENSIONS_THUMBNAILS
from pyams_file.interfaces import IFileInfo, IImageFile, IResponsiveImage
from pyams_file.property import I18nFileProperty
from pyams_i18n.interfaces import II18n, INegotiator
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.registry import get_pyramid_registry, query_utility
from pyams_utils.request import check_request
from pyams_utils.size import get_human_size
from pyams_utils.traversing import get_parent


__docformat__ = 'restructuredtext'

from pyams_content import _


@implementer(IBaseExtFile)
class BaseExtFile(AssociationItem):
    """External file persistent class"""

    title = FieldProperty(IBaseExtFile['title'])
    description = FieldProperty(IBaseExtFile['description'])
    author = FieldProperty(IBaseExtFile['author'])
    language = FieldProperty(IBaseExtFile['language'])
    filename = FieldProperty(IBaseExtFile['filename'])


@adapter_config(required=IBaseExtFile,
                provides=IAssociationInfo)
class BaseExtFileAssociationInfoAdapter(ContextAdapter):
    """Base external file association info adapter"""

    @property
    def pictogram(self):
        """Pictogram getter"""
        return self.context.icon_class

    @property
    def user_title(self):
        """User title getter"""
        request = check_request()
        manager_info = IExtFileManagerInfo(request.root)
        title = II18n(self.context).query_attribute('title', request=request)
        if not title:
            title = self.context.filename
            if '.' in title:
                title, extension = title.rsplit('.', 1)
        prefix = II18n(manager_info).query_attribute('default_title_prefix',
                                                     request=request) or ''
        return f'{prefix} {title}'

    @property
    def user_header(self):
        """User header getter"""
        request = check_request()
        return II18n(self.context).query_attribute('description', request=request)

    @property
    def user_icon(self):
        """User icon getter"""
        filename = self.context.filename
        if filename:
            name, ext = os.path.splitext(filename)
            return f'''<img class="mx-3 align-top"''' \
                   f''' src="/--static--/pyams_file_views/img/16x16/''' \
                   f'''{EXTENSIONS_THUMBNAILS.get(ext, 'unknown.png')}" />'''

    @property
    def inner_title(self):
        """Inner title getter"""
        return self.context.filename or MISSING_INFO

    @property
    def human_size(self):
        """Human size getter"""
        data = II18n(self.context).query_attribute('data')
        if data:
            return get_human_size(data.get_size())
        return MISSING_INFO


def update_properties(extfile):
    """Update missing file properties"""
    request = check_request()
    i18n = query_utility(INegotiator)
    if i18n is not None:
        lang = i18n.server_language
        data = II18n(extfile).get_attribute('data', lang, request)
        if data:
            info = IFileInfo(data)
            info.title = II18n(extfile).get_attribute('title', lang, request)
            info.description = II18n(extfile).get_attribute('description', lang, request)
            if not extfile.filename:
                extfile.filename = info.filename
            else:
                info.filename = extfile.filename
            info.language = extfile.language


@subscriber(IObjectAddedEvent, context_selector=IBaseExtFile)
def handle_added_extfile(event):
    """Handle added external file"""
    # update inner file properties
    extfile = event.object
    update_properties(extfile)
    # notify content modification
    content = get_parent(extfile, IWfSharedContent)
    if content is not None:
        get_pyramid_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectModifiedEvent, context_selector=IBaseExtFile)
def handle_modified_extfile(event):
    """Handle modified external file"""
    # update inner file properties
    extfile = event.object
    update_properties(extfile)
    # notify content modification
    content = get_parent(extfile, IWfSharedContent)
    if content is not None:
        get_pyramid_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectRemovedEvent, context_selector=IBaseExtFile)
def handle_removed_extfile(event):
    """Handle removed external file"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_pyramid_registry().notify(ObjectModifiedEvent(content))


@factory_config(IExtFile)
class ExtFile(BaseExtFile):
    """Generic external file persistent class"""

    icon_class = EXTFILE_ICON_CLASS
    icon_hint = EXTFILE_ICON_HINT

    data = I18nFileProperty(IExtFile['data'])


@factory_config(IExtImage)
class ExtImage(BaseExtFile):
    """External image persistent class"""

    icon_class = EXTIMAGE_ICON_CLASS
    icon_hint = EXTIMAGE_ICON_HINT

    _data = I18nFileProperty(IExtImage['data'])

    @property
    def data(self):
        """Data getter"""
        return self._data

    @data.setter
    def data(self, value):
        """Data setter"""
        self._data = value
        for data in self._data.values():
            if IImageFile.providedBy(data):
                alsoProvides(data, IResponsiveImage)

    @data.deleter
    def data(self):
        """Data deleter"""
        del self._data


@factory_config(IExtVideo)
class ExtVideo(BaseExtFile):
    """External video file persistent class"""

    icon_class = EXTVIDEO_ICON_CLASS
    icon_hint = EXTVIDEO_ICON_HINT

    data = I18nFileProperty(IExtVideo['data'])


@factory_config(IExtAudio)
class ExtAudio(BaseExtFile):
    """External audio file persistent class"""

    icon_class = EXTAUDIO_ICON_CLASS
    icon_hint = EXTAUDIO_ICON_HINT

    data = I18nFileProperty(IExtAudio['data'])



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from zope.schema import Bool, Choice, Int, List, Set

from pyams_content.component.paragraph import CONTENT_PARAGRAPHS_VOCABULARY, \
    PARAGRAPH_FACTORIES_VOCABULARY
from pyams_i18n.schema import I18nTextLineField
from pyams_portal.interfaces import IPortletSettings
from pyams_sequence.schema import InternalReferenceField

from pyams_content import _


class IParagraphContainerPortletSettings(IPortletSettings):
    """Paragraph container portlet settings"""

    title = I18nTextLineField(title=_("Title"),
                              required=False)

    reference = InternalReferenceField(title=_("Paragraphs source"),
                                       description=_("If a source is specified, it will be used "
                                                     "as paragraphs source instead of current "
                                                     "context"),
                                       required=False)

    button_label = I18nTextLineField(title=_("Button label"),
                                     description=_("If a source is specified, you can create a "
                                                   "link to this content using a button with "
                                                   "this label"),
                                     required=False)

    paragraphs = List(title=_("Selected paragraphs"),
                      description=_("List of selected paragraphs; an empty selection means that "
                                    "all paragraphs will be selectable by following filters; "
                                    "otherwise, this selection will have priority"),
                      value_type=Choice(vocabulary=CONTENT_PARAGRAPHS_VOCABULARY),
                      required=False)

    factories = Set(title=_("Paragraph types"),
                    description=_("Select list of paragraph types you want to include; an empty "
                                  "selection means that all paragraphs types will be selected"),
                    value_type=Choice(vocabulary=PARAGRAPH_FACTORIES_VOCABULARY),
                    required=False)

    anchors_only = Bool(title=_("Anchors only"),
                        description=_("If 'yes', only paragraphs set as 'anchors' will be "
                                      "selected"),
                        required=True,
                        default=False)

    exclude_anchors = Bool(title=_("Exclude anchors"),
                           description=_("If 'yes', paragraphs set as 'anchors' will be "
                                         "excluded; take care to not activate this option and "
                                         "the previous one simultaneously, or no paragraph will "
                                         "be displayed!"),
                           required=True,
                           default=False)

    limit = Int(title=_("Paragraphs count limit"),
                description=_("If specified, the number of displayed paragraphs will be limited "
                              "to this number"),
                required=False)

    display_navigation_links = Bool(title=_("Display navigation links"),
                                    description=_("If 'no', navigation links to previous and "
                                                  "next contents will not be displayed"),
                                    required=True,
                                    default=True)


class IParagraphNavigationPortletSettings(IPortletSettings):
    """Paragraphs container navigation settings interface"""

    paragraphs = List(title=_("Selected paragraphs"),
                      description=_("List of paragraphs selected for navigation; an empty "
                                    "selection means that all paragraphs will be selectable by "
                                    "following filters; otherwise, this selection will have "
                                    "priority"),
                      value_type=Choice(vocabulary=CONTENT_PARAGRAPHS_VOCABULARY),
                      required=False)

    factories = Set(title=_("Paragraph types"),
                    description=_("Select list of paragraph types you want to use for "
                                  "navigation; an empty selection means that all paragraphs "
                                  "types will be selected"),
                    value_type=Choice(vocabulary=PARAGRAPH_FACTORIES_VOCABULARY),
                    required=False)

    anchors_only = Bool(title=_("Anchors only"),
                        description=_("If 'no', all paragraphs will be used as navigation "
                                      "anchors"),
                        required=True,
                        default=True)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.location.interfaces import ISublocations
from zope.traversing.interfaces import ITraversable

from pyams_content.feature.filter.interfaces import FILTER_CONTAINER_ANNOTATION_KEY, IFilterProcessor, \
    IFiltersContainer, IFiltersContainerTarget
from pyams_portal.interfaces import MANAGE_TEMPLATE_PERMISSION
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config, get_annotation_adapter
from pyams_utils.container import BTreeOrderedContainer
from pyams_utils.factory import factory_config

__docformat__ = 'restructuredtext'


#
# Filters container
#

@factory_config(IFiltersContainer)
class FilterContainer(BTreeOrderedContainer):
    """Filter container"""

    def get_visible_filters(self):
        yield from filter(lambda x: x.visible, self.values())

    def get_processed_filters(self, context, request, aggregations):
        registry = request.registry
        for filter in self.get_visible_filters():
            processor = registry.queryMultiAdapter((filter, request, self),
                                                   IFilterProcessor)
            if processor is not None:
                yield processor.process(aggregations)


@adapter_config(required=IFiltersContainer,
                provides=IViewContextPermissionChecker)
class FilterContainerPermissionChecker(ContextAdapter):
    """Filter container permission checker"""

    edit_permission = MANAGE_TEMPLATE_PERMISSION


@adapter_config(required=IFiltersContainerTarget,
                provides=IFiltersContainer)
def filter_container(context):
    """Filter container adapter"""
    return get_annotation_adapter(context, FILTER_CONTAINER_ANNOTATION_KEY,
                                  IFiltersContainer, name='++filter++')


@adapter_config(name='filter',
                required=IFiltersContainerTarget,
                provides=ITraversable)
class FilterContainerTraverser(ContextAdapter):
    """Filter container traverser"""

    def traverse(self, name, furtherPath=None):
        container = IFiltersContainer(self.context, None)
        if name:
            return container.get(name)
        return container


@adapter_config(name='filter',
                required=IFiltersContainerTarget,
                provides=ISublocations)
class FilterContainerSublocations(ContextAdapter):
    """Filter container sublocations"""

    def sublocations(self):
        return IFiltersContainer(self.context).values()



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.illustration.skin module

This module provides base illustrations adapters.
"""

from zope.interface import Interface

from pyams_content.component.illustration import IBaseIllustrationTarget, IIllustration, \
    IIllustrationTargetBase, ILinkIllustration, ILinkIllustrationTarget
from pyams_content.component.links import IInternalLink
from pyams_content.interfaces import IBaseContent
from pyams_content.shared.common import ISharedContent
from pyams_content.skin.interfaces import IContentBannerIllustration, IContentIllustration, \
    IContentNavigationIllustration
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import HIDDEN_RENDERER_NAME
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.tales import ITALESExtension


#
# Illustrations adapters
#

@adapter_config(required=(IIllustrationTargetBase, IPyAMSLayer),
                provides=IContentIllustration)
def base_content_illustration(context, request):
    """Base content illustration adapter"""
    illustration = IIllustration(context, None)
    if illustration and illustration.has_data():
        return illustration
    return None


@adapter_config(name='pyams_illustration',
                required=(Interface, Interface, Interface),
                provides=ITALESExtension)
class PyAMSIllustrationTALESExtension(ContextRequestViewAdapter):
    """PyAMS illustration TALES extension"""

    def render(self, context=None, name=''):
        if context is None:
            context = self.context
        return self.request.registry.queryMultiAdapter((context, self.request),
                                                       IContentIllustration,
                                                       name=name)


#
# Navigation illustration adapters
#

@adapter_config(required=(IBaseIllustrationTarget, IPyAMSLayer),
                provides=IContentNavigationIllustration)
def base_content_navigation_illustration(context, request):
    """Default content navigation illustration adapter"""
    illustration = ILinkIllustration(context, None)
    if not (illustration and illustration.has_data()):
        illustration = IIllustration(context, None)
        if IIllustration.providedBy(illustration) and \
                (illustration.renderer == HIDDEN_RENDERER_NAME):
            illustration = None
    if illustration and illustration.has_data():
        return illustration
    return None


@adapter_config(required=(IInternalLink, IPyAMSLayer),
                provides=IContentNavigationIllustration)
def internal_link_navigation_illustration(context, request):
    """Internal link navigation illustration adapter"""
    target = context.get_target()
    if target is not None:
        illustration = request.registry.queryMultiAdapter((target, request),
                                                          IContentNavigationIllustration)
        if illustration and illustration.has_data():
            return illustration
    return None


@adapter_config(required=(ILinkIllustrationTarget, IPyAMSLayer),
                provides=IContentNavigationIllustration)
def link_content_navigation_illustration(context, request):
    """Content navigation illustration adapter for basic link illustration targets"""
    illustration = ILinkIllustration(context, None)
    if illustration and illustration.has_data():
        return illustration


@adapter_config(required=(ISharedContent, IPyAMSLayer),
                provides=IContentNavigationIllustration)
def shared_content_navigation_illustration(context, request):
    """Shared content illustration factory"""
    version = context.visible_version
    if version is not None:
        return request.registry.queryMultiAdapter((version, request),
                                                  IContentNavigationIllustration)
    return None


@adapter_config(name='pyams_navigation_illustration',
                required=(Interface, Interface, Interface),
                provides=ITALESExtension)
class PyAMSNavigationIllustrationTALESExtension(ContextRequestViewAdapter):
    """PyAMS navigation illustration TALES extension"""

    def render(self, context=None, name=''):
        if context is None:
            context = self.context
        return self.request.registry.queryMultiAdapter((context, self.request),
                                                       IContentNavigationIllustration,
                                                       name=name)


#
# Banner illustration adapters
#

@adapter_config(required=(ISiteRoot, IPyAMSLayer),
                provides=IContentBannerIllustration)
@adapter_config(context=(IBaseContent, IPyAMSLayer),
                provides=IContentBannerIllustration)
def base_content_banner_illustration(context, request):
    """Base content banner illustration adapter"""
    illustration = IIllustration(context, None)
    if illustration and illustration.has_data() and (illustration.renderer != HIDDEN_RENDERER_NAME):
        return illustration
    return None


@adapter_config(name='pyams_banner_illustration',
                required=(Interface, Interface, Interface),
                provides=ITALESExtension)
class PyAMSBannerIllustrationTALESExtension(ContextRequestViewAdapter):
    """PyAMS banner illustration TALES extension"""

    def render(self, context=None, name=''):
        if context is None:
            context = self.context
        return self.request.registry.queryMultiAdapter((context, self.request),
                                                       IContentBannerIllustration,
                                                       name=name)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.extfile.zmi module

This module provides management components for external files.
"""

from zope.interface import Interface, implementer

from pyams_content.component.association import IAssociationContainer, IAssociationContainerTarget
from pyams_content.component.association.zmi import AssociationItemAddFormMixin, \
    AssociationItemAddMenuMixin, IAssociationsTable
from pyams_content.component.extfile import ExtAudio, ExtFile, ExtImage, ExtVideo, IExtAudio, \
    IExtFile, IExtImage, IExtVideo
from pyams_content.component.extfile.interfaces import IExtFileContainerTarget
from pyams_content.component.extfile.zmi.interfaces import IExtFileAddForm, IExtFileEditForm
from pyams_content.component.extfile.zmi.widget import I18nExtFileTitleFieldWidget
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.security import ProtectedViewObjectMixin
from pyams_skin.viewlet.menu import MenuDivider, MenuItem
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.interfaces import IAdminLayer, IObjectHint, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager
from pyams_zmi.utils import get_object_hint, get_object_label


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(IExtFile, IAdminLayer, Interface),
                provides=IObjectHint)
def external_file_hint(context, request, view):  # pylint: disable=unused-argument
    """External file hint getter"""
    return request.localizer.translate(context.icon_hint)


@implementer(IExtFileAddForm)
class ExtFileAddFormMixin(AssociationItemAddFormMixin):
    """External file add form mixin class"""

    legend = _("New file properties")
    modal_class = 'modal-xl'


@adapter_config(required=(IAssociationContainer, IAdminLayer, IExtFileAddForm),
                provides=IFormTitle)
def base_extfile_add_form_title(context, request, form):
    """Base external file add form title getter"""
    parent = get_parent(context, IAssociationContainerTarget)
    hint = get_object_hint(parent, request, form)
    label = get_object_label(parent, request, form)
    return TITLE_SPAN_BREAK.format(hint, label)


@implementer(IExtFileEditForm)
class ExtFileEditFormMixin:
    """External file edit form mixin class"""

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        return translate(_("{}: {}")).format(
            translate(self.context.icon_hint),
            get_object_label(self.context, self.request, self))

    legend = _("External file properties")
    modal_class = 'modal-xl'


@adapter_config(required=(IExtFile, IAdminLayer, IExtFileEditForm),
                provides=IFormTitle)
def extfile_edit_form_title(context, request, form):
    """External file edit form title"""
    parent = get_parent(context, IAssociationContainerTarget)
    hint = get_object_hint(parent, request, form)
    label = get_object_label(parent, request, form)
    return TITLE_SPAN_BREAK.format(
        hint, label)


#
# External files forms
#

@viewlet_config(name='add-extfile.divider',
                context=IExtFileContainerTarget, layer=IAdminLayer, view=IAssociationsTable,
                manager=IContextAddingsViewletManager, weight=69)
class ExtFileAddMenuDivider(ProtectedViewObjectMixin, MenuDivider):
    """External file add menu divider"""


@viewlet_config(name='add-extfile.menu',
                context=IExtFileContainerTarget, layer=IAdminLayer, view=IAssociationsTable,
                manager=IContextAddingsViewletManager, weight=70)
class ExtFileAddMenu(ProtectedViewObjectMixin, AssociationItemAddMenuMixin, MenuItem):
    """External file add menu"""

    label = ExtFile.icon_hint
    icon_class = ExtFile.icon_class

    href = 'add-extfile.html'


@ajax_form_config(name='add-extfile.html',
                  context=IAssociationContainer, layer=IPyAMSLayer)
class ExtFileAddForm(ExtFileAddFormMixin, AdminModalAddForm):
    """External file add form"""

    subtitle = _("New external file")
    legend = _("New external file properties")

    fields = Fields(IExtFile).select('data', 'filename', 'title', 'description',
                                     'author', 'language')
    fields['title'].widget_factory = I18nExtFileTitleFieldWidget

    content_factory = IExtFile


@ajax_form_config(name='properties.html',
                  context=IExtFile, layer=IPyAMSLayer)
class ExtFileEditForm(ExtFileEditFormMixin, AdminModalEditForm):
    """External file properties edit form"""

    fields = Fields(IExtFile).select('data', 'filename', 'title', 'description',
                                     'author', 'language')
    fields['title'].widget_factory = I18nExtFileTitleFieldWidget


#
# External images forms
#

@viewlet_config(name='add-extimage.menu',
                context=IExtFileContainerTarget, layer=IAdminLayer, view=IAssociationsTable,
                manager=IContextAddingsViewletManager, weight=75)
class ExtImageAddMenu(ProtectedViewObjectMixin, AssociationItemAddMenuMixin, MenuItem):
    """External image add menu"""

    label = ExtImage.icon_hint
    icon_class = ExtImage.icon_class

    href = 'add-extimage.html'


@ajax_form_config(name='add-extimage.html',
                  context=IAssociationContainer, layer=IPyAMSLayer)
class ExtImageAddForm(ExtFileAddFormMixin, AdminModalAddForm):
    """External image add form"""

    subtitle = _("New external image")
    legend = _("New external image properties")

    fields = Fields(IExtImage).select('data', 'filename', 'title', 'description',
                                      'author', 'language')
    fields['title'].widget_factory = I18nExtFileTitleFieldWidget

    content_factory = IExtImage


@ajax_form_config(name='properties.html',
                  context=IExtImage, layer=IPyAMSLayer)
class ExtImageEditForm(ExtFileEditFormMixin, AdminModalEditForm):
    """External image properties edit form"""

    fields = Fields(IExtImage).select('data', 'filename', 'title', 'description',
                                      'author', 'language')
    fields['title'].widget_factory = I18nExtFileTitleFieldWidget


#
# External videos forms
#

@viewlet_config(name='add-extvideo.menu',
                context=IExtFileContainerTarget, layer=IAdminLayer, view=IAssociationsTable,
                manager=IContextAddingsViewletManager, weight=80)
class ExtVideoAddMenu(ProtectedViewObjectMixin, AssociationItemAddMenuMixin, MenuItem):
    """External video add menu"""

    label = ExtVideo.icon_hint
    icon_class = ExtVideo.icon_class

    href = 'add-extvideo.html'


@ajax_form_config(name='add-extvideo.html',
                  context=IAssociationContainer, layer=IPyAMSLayer)
class ExtVideoAddForm(ExtFileAddFormMixin, AdminModalAddForm):
    """External video add form"""

    subtitle = _("New external video")
    legend = _("New external video properties")

    fields = Fields(IExtVideo).select('data', 'filename', 'title', 'description',
                                      'author', 'language')
    fields['title'].widget_factory = I18nExtFileTitleFieldWidget

    content_factory = IExtVideo


@ajax_form_config(name='properties.html',
                  context=IExtVideo, layer=IPyAMSLayer)
class ExtVideoEditForm(ExtFileEditFormMixin, AdminModalEditForm):
    """External video properties edit form"""

    fields = Fields(IExtVideo).select('data', 'filename', 'title', 'description',
                                      'author', 'language')
    fields['title'].widget_factory = I18nExtFileTitleFieldWidget


#
# External audios forms
#

@viewlet_config(name='add-extaudio.menu',
                context=IExtFileContainerTarget, layer=IAdminLayer, view=IAssociationsTable,
                manager=IContextAddingsViewletManager, weight=85)
class ExtAudioAddMenu(ProtectedViewObjectMixin, AssociationItemAddMenuMixin, MenuItem):
    """External audio add menu"""

    label = ExtAudio.icon_hint
    icon_class = ExtAudio.icon_class

    href = 'add-extaudio.html'


@ajax_form_config(name='add-extaudio.html',
                  context=IAssociationContainer, layer=IPyAMSLayer)
class ExtAudioAddForm(ExtFileAddFormMixin, AdminModalAddForm):
    """External audio add form"""

    subtitle = _("New external audio")
    legend = _("New external audio properties")

    fields = Fields(IExtAudio).select('data', 'filename', 'title', 'description',
                                      'author', 'language')
    fields['title'].widget_factory = I18nExtFileTitleFieldWidget

    content_factory = IExtAudio


@ajax_form_config(name='properties.html',
                  context=IExtAudio, layer=IPyAMSLayer)
class ExtAudioEditForm(ExtFileEditFormMixin, AdminModalEditForm):
    """External audio properties edit form"""

    fields = Fields(IExtAudio).select('data', 'filename', 'title', 'description',
                                      'author', 'language')
    fields['title'].widget_factory = I18nExtFileTitleFieldWidget



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.interfaces module

"""

from zope.annotation import IAttributeAnnotatable
from zope.container.constraints import containers, contains
from zope.container.interfaces import IOrderedContainer
from zope.interface import Interface
from zope.schema import Bool, Choice, TextLine

from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.component.paragraph.interfaces import ParagraphRendererChoice
from pyams_content.feature.renderer import IRenderedContent
from pyams_file.schema import AudioField, MediaField
from pyams_i18n.schema import I18nTextField, I18nTextLineField


__docformat__ = 'restructuredtext'

from pyams_content import _  # pylint: disable=ungrouped-imports


GALLERY_CONTAINER_KEY = 'pyams_content.gallery'
GALLERY_RENDERERS = 'pyams.gallery.renderers'


class IGalleryItem(Interface):
    """Gallery item interface"""

    containers('.IGallery')


class IGalleryFile(IGalleryItem):
    """Gallery file interface"""

    data = MediaField(title=_("Image or video data"),
                      description=_("Image or video content"),
                      required=True)

    title = I18nTextLineField(title=_("Legend"),
                              required=False)

    alt_title = I18nTextLineField(title=_("Accessibility title"),
                                  description=_("Alternate title used to describe media content"),
                                  required=False)

    description = I18nTextField(title=_("Associated text"),
                                description=_("Media description can be displayed in "
                                              "front-office templates"),
                                required=False)

    author = TextLine(title=_("Author"),
                      description=_("Name of media's author"),
                      required=False)

    sound = AudioField(title=_("Audio data"),
                       description=_("Sound file associated with the current media"),
                       required=False)

    sound_title = I18nTextLineField(title=_("Sound title"),
                                    description=_("Title of associated sound file"),
                                    required=False)

    sound_description = I18nTextField(title=_("Sound description"),
                                      description=_("Short description of associated sound file"),
                                      required=False)

    visible = Bool(title=_("Visible media?"),
                   description=_("If 'no', this media won't be displayed in front office"),
                   required=True,
                   default=True)


class IGalleryContainer(IOrderedContainer):
    """Base gallery container interface"""

    contains(IGalleryItem)

    def append(self, item, notify=True):
        """Append new file to gallery

        @param item: the media object to append
        @param boolean notify: if 'False', the item value object is pre-located so that adding
            events are not notified
        """

    def get_visible_medias(self):
        """Get iterator over visible medias"""

    def get_visible_images(self):
        """Get iterator over visible images"""


class IBaseGallery(IGalleryContainer, IRenderedContent):
    """Base gallery interface"""

    renderer = Choice(title=_("Gallery renderer"),
                      description=_("Presentation template used for this gallery"),
                      vocabulary=GALLERY_RENDERERS,
                      default='default')


class IGallery(IBaseGallery):
    """Gallery interface"""

    title = I18nTextLineField(title=_("Title"),
                              description=_("Gallery title, as shown in front-office"),
                              required=False)

    description = I18nTextField(title=_("Description"),
                                description=_("Gallery description displayed by front-office "
                                              "template"),
                                required=False)


class IGalleryTarget(IAttributeAnnotatable):
    """Gallery container target marker interface"""


GALLERY_PARAGRAPH_TYPE = 'gallery'
GALLERY_PARAGRAPH_NAME = _("Medias gallery")
GALLERY_PARAGRAPH_RENDERERS = 'PyAMS_content.paragraph.gallery.renderers'
GALLERY_PARAGRAPH_ICON_CLASS = 'fas fa-images'


class IGalleryParagraph(IBaseGallery, IBaseParagraph):
    """Gallery paragraph"""

    renderer = ParagraphRendererChoice(description=_("Presentation template used for gallery"),
                                       renderers=GALLERY_PARAGRAPH_RENDERERS)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.zmi.folder module

"""

from zope.interface import Interface
from zope.intid import IIntIds
from zope.schema import Int, Text

from pyams_content.interfaces import IBaseContent, MANAGE_SITE_PERMISSION
from pyams_content.shared.common.interfaces import IBaseSharedTool
from pyams_content.zmi.properties import PropertiesEditForm
from pyams_content.shared.site.interfaces import ISiteContainer, ISiteFolder, ISiteManager
from pyams_content.shared.site.zmi.interfaces import ISiteTreeTable
from pyams_content.shared.site.zmi.widget.folder import SiteManagerFoldersSelectorFieldWidget
from pyams_form.ajax import AJAXFormRenderer, ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IGroup
from pyams_i18n.interfaces import II18n, INegotiator
from pyams_i18n.schema import I18nTextLineField
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.viewlet import IBreadcrumbItem
from pyams_skin.viewlet.breadcrumb import BreadcrumbItem
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_utils.unicode import translate_string
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, FormGroupSwitcher
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager, IPropertiesMenu, \
    ISiteManagementMenu
from pyams_zmi.table import TableElementEditor
from pyams_zmi.utils import get_object_label
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(ISiteFolder, IAdminLayer, Interface),
                provides=IObjectLabel)
def site_folder_label(context, request, view):
    """Site folder label"""
    return II18n(context).query_attribute('title', request=request)


@viewlet_config(name='add-site-folder.menu',
                context=ISiteContainer, layer=IAdminLayer, view=ISiteTreeTable,
                manager=IContextAddingsViewletManager, weight=10,
                permission=MANAGE_SITE_PERMISSION)
class SiteFolderAddMenu(MenuItem):
    """Site folder add menu"""

    label = _("Add site folder...")
    icon_class = 'far fa-folder'

    href = 'add-site-folder.html'
    modal_target = True


class ISiteFolderAddFormFields(Interface):
    """Site folder add form fields interface"""

    title = I18nTextLineField(title=_("Title"),
                              description=_("Visible label used to display content"),
                              required=True)

    parent = Int(title=_("Parent"),
                 description=_("Folder's parent"),
                 required=True)

    notepad = Text(title=_("Notepad"),
                   description=_("Internal information to be known about this content"),
                   required=False)


@ajax_form_config(name='add-site-folder.html',
                  context=ISiteContainer, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_PERMISSION)
class SiteFolderAddForm(AdminModalAddForm):
    """Site folder add form"""

    subtitle = _("New site folder")
    legend = _("New site folder properties")

    fields = Fields(ISiteFolderAddFormFields)
    fields['parent'].widget_factory = SiteManagerFoldersSelectorFieldWidget
    content_factory = ISiteFolder

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        if 'parent' in self.widgets:
            self.widgets['parent'].permission = MANAGE_SITE_PERMISSION

    def update_content(self, content, data):
        data = data.get(self, data)
        # initialize new folder
        content.title = data['title']
        content.short_name = data['title']
        content.notepad = data['notepad']
        intids = get_utility(IIntIds)
        parent = intids.queryObject(data.get('parent'))
        if parent is not None:
            negotiator = get_utility(INegotiator)
            title = II18n(content).get_attribute('title', lang=negotiator.server_language)
            name = translate_string(title, force_lower=True, spaces='-')
            if name in parent:
                index = 1
                new_name = '{name}-{index:02}'.format(name=name, index=index)
                while new_name in parent:
                    index += 1
                    new_name = '{name}-{index:02}'.format(name=name, index=index)
                name = new_name
            parent[name] = content

    def add(self, content):
        pass


@adapter_config(required=(ISiteContainer, IAdminLayer, SiteFolderAddForm),
                provides=IFormTitle)
def site_folder_add_form_title(context, request, form):
    """Site folder add form title"""
    manager = get_parent(context, ISiteManager)
    if manager is context:
        return get_object_label(manager, request, form)
    return TITLE_SPAN_BREAK.format(
        get_object_label(manager, request, form),
        get_object_label(context, request, form))


@adapter_config(required=(ISiteFolder, IAdminLayer, Interface),
                provides=ITableElementEditor)
class SiteFolderTableElementEditor(TableElementEditor):
    """Site folder table element editor"""

    view_name = 'admin'
    modal_target = False


@adapter_config(required=(ISiteFolder, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class SiteFolderBreadcrumbs(BreadcrumbItem):
    """Site folder breadcrumb item"""

    @property
    def label(self):
        return II18n(self.context).query_attribute('short_name', request=self.request)

    view_name = 'admin'


@viewletmanager_config(name='properties.menu',
                       context=ISiteFolder, layer=IAdminLayer,
                       manager=ISiteManagementMenu, weight=10,
                       provides=IPropertiesMenu,
                       permission=VIEW_SYSTEM_PERMISSION)
class SiteFolderPropertiesMenu(NavigationMenuItem):
    """Site folder properties menu"""

    label = _("Properties")
    icon_class = 'fas fa-edit'
    href = '#properties.html'


@ajax_form_config(name='properties.html',
                  context=ISiteFolder, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class SiteFolderPropertiesEditForm(PropertiesEditForm):
    """Site folder properties edit form"""

    title = _("Site folder properties")
    legend = _("Main folder properties")

    fields = Fields(ISiteFolder).select('title', 'short_name', 'header', 'description') + \
        Fields(IBaseSharedTool).select('shared_content_workflow') + \
        Fields(ISiteFolder).select('notepad')


@adapter_config(required=(ISiteFolder, IAdminLayer, SiteFolderPropertiesEditForm),
                provides=IAJAXFormRenderer)
class SiteFolderPropertiesEditFormRenderer(AJAXFormRenderer):
    """Site folder properties edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        if 'title' in changes.get(IBaseContent, ()):
            return {
                'status': 'reload',
                'message': self.request.localizer.translate(self.form.success_message)
            }
        return super().render(changes)


@adapter_config(name='navigation',
                required=(ISiteFolder, IAdminLayer, SiteFolderPropertiesEditForm),
                provides=IGroup)
class SiteFolderPropertiesEditFormNavigationGroup(FormGroupSwitcher):
    """Site folder properties edit form navigation group"""

    legend = _("Navigation properties")
    weight = 10

    fields = Fields(ISiteFolder).select('visible_in_list', 'navigation_title', 'navigation_mode')



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_content.feature.header.interfaces import IPageHeaderTitle
from pyams_content.shared.common import IBaseSharedTool, IWfSharedContent
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_utils.adapter import adapter_config

__docformat__ = 'restructuredtext'


@adapter_config(required=(IBaseSharedTool, IPyAMSLayer),
                provides=IPageHeaderTitle)
def base_shared_tool_page_title(context, request):
    """Base shared tool page title getter"""
    return II18n(context).query_attribute('title', request=request)


@adapter_config(required=(IWfSharedContent, IPyAMSLayer),
                provides=IPageHeaderTitle)
def shared_content_page_title(context, request):
    """Shared content page title getter"""
    return II18n(context).query_attribute('title', request=request)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.association.zmi.paragraph module

"""

from pyams_content.component.association.interfaces import ASSOCIATION_PARAGRAPH_ICON_CLASS, \
    ASSOCIATION_PARAGRAPH_NAME, ASSOCIATION_PARAGRAPH_TYPE, IAssociationParagraph
from pyams_content.component.paragraph import IParagraphContainer, IParagraphContainerTarget
from pyams_content.component.paragraph.zmi import BaseParagraphAddForm, BaseParagraphAddMenu, \
    IParagraphContainerBaseTable
from pyams_form.ajax import ajax_form_config
from pyams_layer.interfaces import IPyAMSLayer
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager


__docformat__ = 'restructuredtext'


@viewlet_config(name='add-association-paragraph.menu',
                context=IParagraphContainerTarget, layer=IAdminLayer,
                view=IParagraphContainerBaseTable,
                manager=IContextAddingsViewletManager, weight=90)
class AssociationParagraphAddMenu(BaseParagraphAddMenu):
    """Association paragraph add menu"""

    label = ASSOCIATION_PARAGRAPH_NAME
    icon_class = ASSOCIATION_PARAGRAPH_ICON_CLASS

    factory_name = ASSOCIATION_PARAGRAPH_TYPE
    href = 'add-association-paragraph.html'


@ajax_form_config(name='add-association-paragraph.html',
                  context=IParagraphContainer, layer=IPyAMSLayer)
class AssociationParagraphAddForm(BaseParagraphAddForm):
    """Association paragraph add form"""

    content_factory = IAssociationParagraph



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.portlet module

This module defines interfaces of portlets which can be used to display a medias gallery.
"""

from zope.schema import Bool

from pyams_content.component.gallery.interfaces import IGalleryContainer
from pyams_i18n.schema import I18nTextLineField
from pyams_portal.interfaces import IPortletSettings


__docformat__ = 'restructuredtext'

from pyams_content import _


class IGalleryPortletSettings(IPortletSettings, IGalleryContainer):
    """Gallery portlet settings interface"""

    title = I18nTextLineField(title=_("Title"),
                              description=_("Main component title"),
                              required=False)

    use_context_gallery = Bool(title=_("Use context gallery"),
                               description=_("If 'yes' and if the displayed context supports "
                                             "medias galleries, this gallery will be used to "
                                             "get medias"),
                               required=True,
                               default=False)

    def get_visible_medias(self, context):
        """Get iterator over visible medias"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.history.zmi.viewlet module

This module provides a small content provider which can be used to display
previous history comments.
"""

__docformat__ = 'restructuredtext'

from pyams_content.feature.history import IHistoryContainer
from pyams_security.utility import get_principal
from pyams_template.template import template_config
from pyams_utils.date import format_datetime
from pyams_utils.timezone import tztime
from pyams_viewlet.viewlet import BaseContentProvider
from pyams_zmi.interfaces import IAdminLayer


@template_config(template='templates/history.pt', layer=IAdminLayer)
class HistoryCommentsContentProvider(BaseContentProvider):
    """Comments history content provider"""

    history = None

    def update(self):
        super().update()
        self.history = IHistoryContainer(self.context, None)

    def render(self, template_name=''):
        if not self.history:
            return ''
        return super().render(template_name)

    def get_principal(self, item):
        return get_principal(self.request, item.principal).title

    @staticmethod
    def get_timestamp(item):
        return format_datetime(tztime(item.timestamp))



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import Interface, implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_content.feature.filter.interfaces import CONTENT_TYPE_FILTER_MODE, ICollectionsFilter, IContentTypesFilter, \
    IFilter, IFilterAggregate, ITagsFilter, IThemesFilter, IThesaurusFilter, ITitleFilter
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import MANAGE_TEMPLATE_PERMISSION
from pyams_security.interfaces import IViewContextPermissionChecker
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_zmi.interfaces import IObjectLabel

__docformat__ = 'restructuredtext'


@implementer(IFilter)
class Filter(Persistent, Contained):
    """Base filter persistent class"""

    filter_type = None

    visible = FieldProperty(IFilter['visible'])
    label = FieldProperty(IFilter['label'])
    display_mode = FieldProperty(IFilter['display_mode'])
    open_state = FieldProperty(IFilter['open_state'])
    displayed_entries = FieldProperty(IFilter['displayed_entries'])
    display_count = FieldProperty(IFilter['display_count'])
    select_placeholder = FieldProperty(IFilter['select_placeholder'])
    sorting_mode = FieldProperty(IFilter['sorting_mode'])


@adapter_config(required=(IFilter, IPyAMSLayer, Interface),
                provides=IObjectLabel)
def filter_label(context, request, view):  # pylint: disable=unused-argument
    """Filter label getter"""
    return II18n(context).get_attribute('label', request=request)


@adapter_config(required=IFilter,
                provides=IViewContextPermissionChecker)
class FilterPermissionChecker(ContextAdapter):
    """Filter permission checker"""

    edit_permission = MANAGE_TEMPLATE_PERMISSION


#
# Content-type filter
#

@factory_config(IContentTypesFilter)
class ContentTypesFilter(Filter):
    """Content-types filter """

    content_mode = FieldProperty(IContentTypesFilter['content_mode'])

    filter_type = CONTENT_TYPE_FILTER_MODE.FACET_LABEL.value


#
# Title filter
#

@factory_config(ITitleFilter)
class TitleFilter(Filter):
    """Title filter"""

    filter_type = 'title'


#
# Thesaurus-based filters
#

@implementer(IThesaurusFilter)
class ThesaurusFilter(Filter):
    """Thesaurus base filter"""

    thesaurus_name = FieldProperty(IThesaurusFilter['thesaurus_name'])
    extract_name = FieldProperty(IThesaurusFilter['extract_name'])


@factory_config(ITagsFilter)
class TagsFilter(ThesaurusFilter):
    """Tags filter"""

    filter_type = 'tags'


@factory_config(ICollectionsFilter)
class CollectionsFilter(ThesaurusFilter):
    """Collections filter"""

    filter_type = 'collections'


@factory_config(IThemesFilter)
class ThemesFilter(ThesaurusFilter):
    """Themes filter"""

    filter_type = 'themes'



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.fields.skin.paragraph module

This module provides components for rendering of form fields paragraphs.
"""

from zope.interface import Interface

from pyams_content.component.fields.interfaces import IFormFieldsParagraph
from pyams_content.component.paragraph.portlet.skin import IParagraphContainerPortletRenderer
from pyams_content.feature.renderer import DefaultContentRenderer, IContentRenderer
from pyams_portal.interfaces import DEFAULT_RENDERER_NAME
from pyams_content.shared.form import IWfForm
from pyams_fields.interfaces import IFormFieldContainerTarget, IFormHandlersInfo
from pyams_fields.skin import FormFieldContainerInputForm
from pyams_fields.skin.interfaces import IFormFieldContainerInputForm
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer, IPyAMSUserLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import ViewContentProvider


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(name='preview.html',
                required=(IFormFieldContainerTarget, IPyAMSLayer),
                provides=IFormFieldContainerInputForm)
class FormFieldContainerInputFormPreview(FormFieldContainerInputForm):
    """Form field container input form fields preview"""


@adapter_config(name=DEFAULT_RENDERER_NAME,
                required=(IFormFieldsParagraph, IPyAMSLayer),
                provides=IContentRenderer)
@template_config(template='templates/paragraph-default.pt', layer=IPyAMSLayer)
class FormFieldsParagraphDefaultRenderer(DefaultContentRenderer):
    """Form fields paragraph default renderer"""

    label = _("Form fields list (default)")

    input_form = None

    def __init__(self, context, request, view=None):
        super().__init__(context, request, view)
        target = get_parent(context, IFormFieldContainerTarget)
        if target is not None:
            form = self.input_form = request.registry.queryMultiAdapter((target, request),
                                                                        IFormFieldContainerInputForm,
                                                                        name=request.view_name)
            if (form is not None) and IWfForm.providedBy(form.context):
                form.legend = II18n(form.context).query_attribute('form_legend', request=request)

    def update(self):
        """Paragraph update"""
        super().update()
        if self.input_form is not None:
            self.input_form.update()

    def render(self, template_name=''):
        """Paragraph render"""
        if self.input_form is None:
            return ''
        return super().render(template_name)


@adapter_config(name='submit.html',
                required=(IFormFieldContainerTarget, IPyAMSUserLayer, Interface),
                provides=IParagraphContainerPortletRenderer)
@template_config(template='templates/form-submit.pt', layer=IPyAMSUserLayer)
class FormSubmitPortletRenderer(ViewContentProvider):
    """Form submit message portlet renderer"""

    use_portlets_cache = False

    @property
    def submit_message(self):
        """Submit message getter"""
        handlers_info = IFormHandlersInfo(self.context, None)
        if handlers_info is None:
            return None
        message = II18n(handlers_info).query_attribute('submit_message', request=self.request)
        reference = self.request.annotations.get('submit.reference')
        if reference:
            message = message.replace('{_reference}', reference)
        return message

    def render(self, template_name=''):
        form = self.view.input_form
        if form.widgets.errors:
            return form.render()
        return super().render(template_name)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.events import subscriber
from zope.interface import Invalid

from pyams_content.feature.filter.interfaces import IAggregatedPortletRendererSettings
from pyams_content.shared.view.portlet.interfaces import IViewItemsPortletSettings
from pyams_form.interfaces.form import IDataExtractedEvent
from pyams_portal.interfaces import IPortletRenderer
from pyams_portal.zmi.portlet import PortletConfigurationEditForm
from pyams_utils.factory import create_object

__docformat__ = 'restructuredtext'

from pyams_content import _


@subscriber(IDataExtractedEvent, form_selector=PortletConfigurationEditForm)
def handle_view_items_portlet_settings_configuration_data(event):
    """Handle view items portlet settings configuration data extraction"""
    form = event.form
    settings = IViewItemsPortletSettings(form.form_content, None)
    if settings is None:
        return
    if len(event.data.get('views', ())) < 2:
        return
    request = form.request
    registry = request.registry
    renderer_name = event.data.get('renderer')
    renderer = registry.queryMultiAdapter((request.root, request, None, settings),
                                          IPortletRenderer, name=renderer_name)
    if renderer is not None:
        renderer_settings = create_object(renderer.settings_interface)
        if IAggregatedPortletRendererSettings.providedBy(renderer_settings):
            form.widgets.errors += (Invalid(_("Multiple views are not supported with aggregated "
                                              "portlets renderers!")),)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.portlet module

This module defines a portlet used to render view items.
"""

from itertools import islice, tee

from zope.schema.fieldproperty import FieldProperty

from pyams_content.shared.view.interfaces.query import IViewsMerger
from pyams_content.shared.view.portlet.interfaces import IViewItemsAggregates, \
    IViewItemsPortletSettings, SEARCH_EXCLUDED_ITEMS, VIEW_DISPLAY_CONTEXT, VIEW_ITEMS_PORTLET_NAME
from pyams_portal.interfaces import IPortletRendererSettings, PREVIEW_MODE
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_sequence.interfaces import ISequentialIdInfo
from pyams_sequence.workflow import get_last_version, get_sequence_target, get_visible_version
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import DISPLAY_CONTEXT_KEY_NAME
from pyams_utils.list import unique_iter
from pyams_utils.request import check_request, get_annotations

__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(IViewItemsPortletSettings)
class ViewItemsPortletSettings(PortletSettings):
    """View items portlet settings persistent class"""

    title = FieldProperty(IViewItemsPortletSettings['title'])
    views = FieldProperty(IViewItemsPortletSettings['views'])
    views_context = FieldProperty(IViewItemsPortletSettings['views_context'])
    views_merge_mode = FieldProperty(IViewItemsPortletSettings['views_merge_mode'])
    limit = FieldProperty(IViewItemsPortletSettings['limit'])
    start = FieldProperty(IViewItemsPortletSettings['start'])
    force_canonical_url = FieldProperty(IViewItemsPortletSettings['force_canonical_url'])
    exclude_from_search = FieldProperty(IViewItemsPortletSettings['exclude_from_search'])
    first_page_only = FieldProperty(IViewItemsPortletSettings['first_page_only'])

    def get_views(self):
        """Portlet published views getter"""
        request = check_request()
        preview_mode = get_annotations(request).get(PREVIEW_MODE)
        for oid in self.views or ():
            view = get_sequence_target(oid, state=None)
            if view is not None:
                if preview_mode:
                    view = get_last_version(view)
                else:
                    view = get_visible_version(view)
            if view is not None:
                yield view

    def get_merger(self, request=None):
        """Views merger getter"""
        if request is None:
            request = check_request()
        return request.registry.queryUtility(IViewsMerger, name=self.views_merge_mode)

    def get_items(self, request=None, start=0, length=999, limit=None, ignore_cache=False):
        """Merged view items iterator"""
        if request is None:
            request = check_request()
        if self.views_context == VIEW_DISPLAY_CONTEXT:
            context = request.annotations.get(DISPLAY_CONTEXT_KEY_NAME, request.root)
        else:
            context = request.context
        if not ignore_cache:
            ignore_cache = request.annotations.get(PREVIEW_MODE, False)
        merger = self.get_merger(request)
        if merger is not None:
            renderer_settings = IPortletRendererSettings(self)
            aggregates = IViewItemsAggregates(renderer_settings)
            if aggregates is not None:
                ignore_cache = True
            else:
                aggregates = {}
            get_count = aggregates is not None
            results = merger.get_results(self.get_views(),
                                         context,
                                         ignore_cache=ignore_cache,
                                         request=request,
                                         aggregates=aggregates,
                                         settings=self,
                                         get_count=get_count)
            if get_count:
                count = next(results)
                aggregations = next(results)
            items = islice(unique_iter(results),
                           start + (self.start or 1) - 1,
                           limit or self.limit or 999)
            if (request is not None) and self.exclude_from_search:
                (excluded, items) = tee(items)
                excluded_items = request.annotations.get(SEARCH_EXCLUDED_ITEMS) or set()
                excluded_items |= set((ISequentialIdInfo(item).hex_oid for item in excluded))
                request.annotations[SEARCH_EXCLUDED_ITEMS] = excluded_items
            if get_count:
                yield count
                yield aggregations
            yield from items


@portlet_config(permission=None)
class ViewItemsPortlet(Portlet):
    """View items portlet"""

    name = VIEW_ITEMS_PORTLET_NAME
    label = _("View items")

    settings_factory = IViewItemsPortletSettings
    toolbar_css_class = 'fas fa-search'



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.workflow module

"""

from datetime import datetime, timezone

from pyramid.events import subscriber
from zope.interface import Interface, Invalid
from zope.lifecycleevent import ObjectModifiedEvent

from pyams_content.interfaces import CREATE_VERSION_PERMISSION, MANAGE_CONTENT_PERMISSION, \
    PUBLISH_CONTENT_PERMISSION
from pyams_content.shared.common.interfaces import IBaseSharedTool, ISharedContent, IWfSharedContent, \
    IWfSharedContentRoles
from pyams_content.shared.common.zmi.interfaces import IWorkflowDeleteFormTarget
from pyams_content.workflow import DELETED, DRAFT, PROPOSED, PUBLISHED
from pyams_form.ajax import ajax_form_config
from pyams_form.button import Buttons, handler
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.utility import get_principal
from pyams_skin.interfaces.viewlet import IFormFooterViewletManager, IFormHeaderViewletManager
from pyams_skin.schema.button import CloseButton, SubmitButton
from pyams_skin.viewlet.help import AlertMessage
from pyams_template.template import template_config
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.date import format_datetime
from pyams_utils.text import text_to_html
from pyams_utils.timezone import gmtime, tztime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import Viewlet, viewlet_config
from pyams_workflow.interfaces import IWorkflow, IWorkflowCommentInfo, \
    IWorkflowInfo, IWorkflowPublicationInfo, IWorkflowRequestUrgencyInfo, IWorkflowState, \
    IWorkflowStateLabel, IWorkflowTransitionInfo, IWorkflowVersion, IWorkflowVersions, \
    MANUAL_TRANSITION, NoTransitionAvailableError, SYSTEM_TRANSITION
from pyams_workflow.zmi.transition import WorkflowContentTransitionForm
from pyams_zmi.interfaces import IAdminLayer

__docformat__ = 'restructuredtext'

from pyams_content import _


class ISharedContentWorkflowFormButtons(Interface):
    """Shared content workflow transition form buttons"""

    action = SubmitButton(name='action', title=_("Workflow action"))
    close = CloseButton(name='close', title=_("Cancel"))


class SharedContentWorkflowTransitionForm(WorkflowContentTransitionForm):
    """Shared content workflow transition form"""

    legend = _("Action comment")
    buttons = Buttons(ISharedContentWorkflowFormButtons)

    def update_actions(self):
        super().update_actions()
        if 'action' in self.actions:
            self.actions['action'].title = self.transition.title

    @handler(buttons['action'])
    def handle_delete(self, action):
        """Delete action handler"""
        super().handle_add(self, action)


@adapter_config(required=(IWorkflowVersion, IAdminLayer, SharedContentWorkflowTransitionForm),
                provides=IAJAXFormRenderer)
class SharedContentWorkflowTransitionFormRenderer(ContextRequestViewAdapter):
    """Shared content workflow transition form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        return {
            'status': 'redirect'
        }


@viewlet_config(name='wf-operator-warning',
                context=IWfSharedContent, layer=IAdminLayer,
                view=SharedContentWorkflowTransitionForm,
                manager=IFormHeaderViewletManager, weight=20)
class SharedContentWorkflowOperatorWarning(AlertMessage):
    """Shared content workflow operator warning"""

    def __new__(cls, context, request, view, manager):
        transition = view.transition
        if not transition.user_data.get('show_operator_warning'):
            return None
        state = IWorkflowState(context)
        roles = IWfSharedContentRoles(context)
        if state.state_principal in roles.owner:
            return None
        return AlertMessage.__new__(cls)

    status = 'danger'

    _message = _("WARNING: this request was made by a contributor which is not the owner "
                 "of this content")


@viewlet_config(name='wf-owner-warning',
                context=IWfSharedContent, layer=IAdminLayer,
                view=SharedContentWorkflowTransitionForm,
                manager=IFormHeaderViewletManager, weight=25)
class SharedContentWorkflowOwnerWarning(AlertMessage):
    """Shared content workflow owner warning"""

    def __new__(cls, context, request, view, manager):
        roles = IWfSharedContentRoles(context)
        if request.principal.id in roles.owner:
            return None
        return AlertMessage.__new__(cls)

    status = 'danger'

    _message = _("RECALL: you are not the owner of the content on which you operate!")


#
# Generic transition info
#

@viewlet_config(name='wf-transition-info',
                context=IWfSharedContent, layer=IAdminLayer,
                view=SharedContentWorkflowTransitionForm,
                manager=IFormFooterViewletManager, weight=10)
@template_config(template='templates/wf-transition-info.pt')
class SharedContentWorkflowTransitionFormInfo(Viewlet):
    """Generic workflow transition form info"""

    @property
    def previous_step(self):
        """Previous step getter"""
        translate = self.request.localizer.translate
        workflow = IWorkflow(self.context)
        state = IWorkflowState(self.context)
        registry = self.request.registry
        adapter = registry.queryMultiAdapter((workflow, self.request), IWorkflowStateLabel,
                                             name=state.state)
        if adapter is None:
            adapter = registry.queryAdapter(workflow, IWorkflowStateLabel,
                                            name=state.state)
        if adapter is None:
            adapter = registry.queryAdapter(workflow, IWorkflowStateLabel)
        if adapter is not None:
            state_label = adapter.get_label(self.context, request=self.request)
        else:
            state_label = translate(_("{state} {date}")).format(
                state=translate(workflow.get_state_label(state.state)),
                date=format_datetime(state.state_date, request=self.request))
        return translate(_("{state} | by {principal}")).format(
            state=state_label,
            principal=get_principal(self.request, state.state_principal).title)

    @property
    def previous_message(self):
        """Previous step message"""
        workflow = IWorkflow(self.context)
        state = IWorkflowState(self.context)
        position = 0
        history_item = None
        trigger = SYSTEM_TRANSITION
        while trigger != MANUAL_TRANSITION:
            position -= 1
            history_item = state.history[position]
            if history_item.transition_id:
                try:
                    trigger = workflow.get_transition_by_id(history_item.transition_id).trigger
                except KeyError:
                    trigger = None
            else:
                break
        if history_item:
            return text_to_html((history_item.comment or '').strip())

    @property
    def next_step(self):
        """Next step getter"""
        transition = self.view.transition
        return self.request.localizer.translate(transition.user_data.get('next_step')) \
            if 'next_step' in transition.user_data else None


#
# Publication request
#

@ajax_form_config(name='wf-propose.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=MANAGE_CONTENT_PERMISSION)
class SharedContentPublicationRequestForm(SharedContentWorkflowTransitionForm):
    """Shared content publication request form"""

    legend = _("Publication settings")

    @property
    def fields(self):
        pub_fields = ('publication_effective_date', 'push_end_date',
                      'publication_expiration_date')
        state = IWorkflowState(self.context)
        if state.version_id > 1:
            pub_fields += ('displayed_publication_date',)
        return Fields(IWorkflowTransitionInfo) + \
            Fields(IWorkflowPublicationInfo).select(*pub_fields) + \
            Fields(IWorkflowRequestUrgencyInfo) + \
            Fields(IWorkflowCommentInfo)

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        pub_info = IWorkflowPublicationInfo(self.context)
        widget = self.widgets.get('publication_effective_date')
        if widget is not None:
            widget.required = True
            widget.value = tztime(datetime.now(timezone.utc)).isoformat()
        if pub_info.push_end_date:
            widget = self.widgets.get('push_end_date')
            if widget is not None:
                widget.value = tztime(pub_info.push_end_date).isoformat()
        if pub_info.publication_expiration_date:
            widget = self.widgets.get('publication_expiration_date')
            if widget is not None:
                widget.value = tztime(pub_info.publication_expiration_date).isoformat()
        widget = self.widgets.get('displayed_publication_date')
        if widget is not None:
            widget.value = pub_info.displayed_publication_date

    def create_and_add(self, data):
        data = data.get(self, data)
        pub_info = IWorkflowPublicationInfo(self.context)
        pub_info.publication_effective_date = data.get('publication_effective_date')
        pub_info.push_end_date = data.get('push_end_date')
        pub_info.publication_expiration_date = data.get('publication_expiration_date')
        if 'displayed_publication_date' in data:
            pub_info.displayed_publication_date = data.get('displayed_publication_date')
        return super().create_and_add(data)


@subscriber(IDataExtractedEvent, form_selector=SharedContentPublicationRequestForm)
def handle_publication_request_form_data_extraction(event):
    """Handle publication request form data extraction"""
    data = event.data
    if not data.get('publication_effective_date'):
        event.form.widgets.errors += (Invalid(_("Publication start date is required")),)


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentPublicationRequestForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentPublicationRequestFormHelp(AlertMessage):
    """Shared content publication request form help"""

    status = 'info'

    _message = _("This publication request is going to be transmitted to a content manager.")


#
# Request publication cancel form
#

@ajax_form_config(name='wf-cancel-propose.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=MANAGE_CONTENT_PERMISSION)
class SharedContentPublicationRequestCancelForm(SharedContentWorkflowTransitionForm):
    """Shared content publication request cancel form"""


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentPublicationRequestCancelForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentPublicationRequestCancelFormHelp(AlertMessage):
    """Shared content publication request cancel form help"""

    status = 'info'

    _message = _("If you cancel this publication request, this content will be updatable again.")


#
# Refuse publication form
#

@ajax_form_config(name='wf-refuse.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=PUBLISH_CONTENT_PERMISSION)
class SharedContentPublicationRequestRefuseForm(SharedContentWorkflowTransitionForm):
    """Shared content publication request refuse form"""

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        if 'comment' in self.widgets:
            self.widgets['comment'].required = True


@subscriber(IDataExtractedEvent, form_selector=SharedContentPublicationRequestRefuseForm)
def handle_publication_request_refuse_form_data_extraction(event):
    """Handle publication request refuse form data extraction"""
    comment = (event.data.get('comment') or '').strip()
    if not comment:
        event.form.widgets.errors += (Invalid(_("A comment is required")),)


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentPublicationRequestRefuseForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentPublicationRequestRefuseFormHelp(AlertMessage):
    """Shared content publication request refuse form help"""

    status = 'info'

    _message = _("As a content manager, you considerate that this content can't be published "
                 "'as is'.<br />"
                 "The contributor will be notified of this and will be able to update the "
                 "content before doing a new publication request.")
    message_renderer = 'markdown'


#
# Publication form
#

@ajax_form_config(name='wf-publish.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=PUBLISH_CONTENT_PERMISSION)
class SharedContentPublicationForm(SharedContentWorkflowTransitionForm):
    """Shared content publication form"""

    legend = _("Publication settings")

    @property
    def fields(self):
        pub_fields = ('publication_effective_date', 'push_end_date',
                      'publication_expiration_date')
        state = IWorkflowState(self.context)
        if state.version_id > 1:
            pub_fields += ('displayed_publication_date',)
        return Fields(IWorkflowTransitionInfo) + \
            Fields(IWorkflowPublicationInfo).select(*pub_fields) + \
            Fields(IWorkflowCommentInfo)

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        pub_info = IWorkflowPublicationInfo(self.context)
        widget = self.widgets.get('publication_effective_date')
        if widget is not None:
            widget.required = True
            now = datetime.now(timezone.utc)
            if pub_info.publication_effective_date:
                widget.value = tztime(max(now, pub_info.publication_effective_date)).isoformat()
            else:
                widget.value = tztime(now).isoformat()
        if pub_info.push_end_date:
            widget = self.widgets.get('push_end_date')
            if widget is not None:
                widget.value = tztime(pub_info.push_end_date).isoformat()
        if pub_info.publication_expiration_date:
            widget = self.widgets.get('publication_expiration_date')
            if widget is not None:
                widget.value = tztime(pub_info.publication_expiration_date).isoformat()
        widget = self.widgets.get('displayed_publication_date')
        if widget is not None:
            widget.value = pub_info.displayed_publication_date

    def create_and_add(self, data):
        data = data.get(self, data)
        pub_info = IWorkflowPublicationInfo(self.context)
        pub_info.publication_effective_date = data.get('publication_effective_date')
        pub_info.push_end_date = data.get('push_end_date')
        pub_info.publication_expiration_date = data.get('publication_expiration_date')
        if 'displayed_publication_date' in data:
            pub_info.displayed_publication_date = data.get('displayed_publication_date')
        now = datetime.now(timezone.utc)
        if pub_info.publication_effective_date <= now:
            # immediate publication
            return super().create_and_add(data)
        # delayed publication: we schedule a publication task
        transition = self.transition.user_data.get('prepared_transition')
        if transition is None:
            raise NoTransitionAvailableError(PROPOSED, PUBLISHED)
        info = IWorkflowInfo(self.context)
        info.fire_transition(transition.transition_id, comment=data.get('comment'))
        info.fire_automatic()
        IWorkflowState(self.context).state_urgency = data.get('urgent_request') or False
        self.request.registry.notify(ObjectModifiedEvent(self.context))
        return info


@subscriber(IDataExtractedEvent, form_selector=SharedContentPublicationForm)
def handle_publication_form_data_extraction(event):
    """Handle publication form data extraction"""
    data = event.data
    if not data.get('publication_effective_date'):
        event.form.widgets.errors += (Invalid(_("Publication start date is required")),)


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentPublicationForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentPublicationFormHelp(AlertMessage):
    """Shared content publication form help"""

    status = 'info'

    _message = _("As a manager, you considerate that this content is complete and can be "
                 "published 'as is'.<br />"
                 "This operation will make the content publicly available (except if "
                 "restricted access has been set).")
    message_renderer = 'markdown'


#
# Pre-publication cancel form
#

@ajax_form_config(name='wf-cancel-publish.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=MANAGE_CONTENT_PERMISSION)
class SharedContentPublicationCancelForm(SharedContentWorkflowTransitionForm):
    """Shared content publication cancel form"""


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentPublicationCancelForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentPublicationCancelFormHelp(AlertMessage):
    """Shared content publication cancel form help"""

    status = 'info'

    _message = _("After cancelling the publication, the content will return to proposed "
                 "publication state.")


#
# Publication retire request form
#

@ajax_form_config(name='wf-retiring.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=MANAGE_CONTENT_PERMISSION)
class SharedContentPublicationRetireRequestForm(SharedContentWorkflowTransitionForm):
    """Shared content publication retire request form"""

    fields = Fields(IWorkflowTransitionInfo) + \
        Fields(IWorkflowRequestUrgencyInfo) + \
        Fields(IWorkflowCommentInfo)

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        if 'comment' in self.widgets:
            self.widgets['comment'].required = True


@subscriber(IDataExtractedEvent, form_selector=SharedContentPublicationRetireRequestForm)
def handle_publication_retire_request_form_data_extraction(event):
    """Handle publication retire request form data extraction"""
    comment = (event.data.get('comment') or '').strip()
    if not comment:
        event.form.widgets.errors += (Invalid(_("A comment is required")),)


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentPublicationRetireRequestForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentPublicationRetireRequestFormHelp(AlertMessage):
    """Shared content publication retire request form help"""

    status = 'info'

    _message = _("You considerate that the currently published version should no more be "
                 "publicly visible.<br />"
                 "WARNING: the content will remain visible until a manager validate your "
                 "request.")
    message_renderer = 'markdown'


#
# Publication retire cancel form
#

@ajax_form_config(name='wf-cancel-retiring.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=MANAGE_CONTENT_PERMISSION)
class SharedContentPublicationRetireRequestCancelForm(SharedContentWorkflowTransitionForm):
    """Shared content publication retire request cancel form"""


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentPublicationRetireRequestCancelForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentPublicationRetireRequestCancelFormHelp(AlertMessage):
    """Shared content publication retire request form help"""

    status = 'info'

    _message = _("After cancelling this request, the content will return to it's normal "
                 "published state.")


#
# Publication retire form
#

@ajax_form_config(name='wf-retire.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=PUBLISH_CONTENT_PERMISSION)
class SharedContentPublicationRetireForm(SharedContentWorkflowTransitionForm):
    """Shared content publication retire form"""


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentPublicationRetireForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentPublicationRetireFormHelp(AlertMessage):
    """Shared content publication retire form help"""

    status = 'info'

    _message = _("As a content manager, you considerate that this content should no longer be "
                 "published.<br />"
                 "Retired content won't be visible anymore, but it can be updated and published "
                 "again, or archived.")
    message_renderer = 'markdown'


#
# Publication archive request form
#

@ajax_form_config(name='wf-archiving.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=MANAGE_CONTENT_PERMISSION)
class SharedContentPublicationArchiveRequestForm(SharedContentWorkflowTransitionForm):
    """Shared content publication request archive form"""

    fields = Fields(IWorkflowTransitionInfo) + \
        Fields(IWorkflowRequestUrgencyInfo) + \
        Fields(IWorkflowCommentInfo)


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentPublicationArchiveRequestForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentPublicationArchiveRequestFormHelp(AlertMessage):
    """Shared content publication archive request form help"""

    status = 'info'

    _message = _("This content is already retired and not visible.<br />"
                 "After archiving, it will be backed up but you will not be able to publish "
                 "it again except by creating a new version.")
    message_renderer = 'markdown'


#
# Publication archive cancel form
#

@ajax_form_config(name='wf-cancel-archiving.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=MANAGE_CONTENT_PERMISSION)
class SharedContentPublicationArchiveCancelForm(SharedContentWorkflowTransitionForm):
    """Shared content publication archive request cancel form"""


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentPublicationArchiveCancelForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentPublicationArchiveCancelFormHelp(AlertMessage):
    """Shared content publication archive request cancel form help"""

    status = 'info'

    _message = _("After cancelling this request, the content will return to it's previous "
                 "retired state.")


#
# Publication archive form
#

@ajax_form_config(name='wf-archive.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=PUBLISH_CONTENT_PERMISSION)
class SharedContentPublicationArchiveForm(SharedContentWorkflowTransitionForm):
    """Shared content publication archive form"""


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentPublicationArchiveForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentPublicationArchiveFormHelp(AlertMessage):
    """Shared content publication archive form help"""

    status = 'info'

    _message = _("As a manager, you considerate that this content must be archived.<br />"
                 "After archiving, it will be backed up but you will not be able to publish it "
                 "again except by creating a new version.")
    message_renderer = 'markdown'


#
# Clone form
#

@ajax_form_config(name='wf-clone.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=CREATE_VERSION_PERMISSION)
class SharedContentCloneForm(SharedContentWorkflowTransitionForm):
    """Shared content clone form"""

    def create_and_add(self, data):
        data = data.get(self, data)
        info = IWorkflowInfo(self.context)
        return info.fire_transition_toward(DRAFT, comment=data.get('comment'))


@adapter_config(required=(IWorkflowVersion, IAdminLayer, SharedContentCloneForm),
                provides=IAJAXFormRenderer)
class SharedContentCloneFormRenderer(ContextRequestViewAdapter):
    """Shared content clone form renderer"""

    def render(self, changes):
        if changes is None:
            return None
        return {
            'status': 'redirect',
            'location': absolute_url(changes, self.request, 'admin')
        }


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer,
                view=SharedContentCloneFormRenderer,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentCloneFormRendererHelp(AlertMessage):
    """Shared content clone form help"""

    status = 'info'

    _message = _("You considerate that the currently published must evolve.<br />"
                 "By creating a new version, you can update it's content without impacting "
                 "the currently published one.<br />"
                 "When the new version will be complete, you will be able to make a new "
                 "publication request to replace the currently published version (which will "
                 "be archived automatically).")
    message_renderer = 'markdown'


#
# Delete form
#

@ajax_form_config(name='wf-delete.html',
                  context=IWfSharedContent, layer=IPyAMSLayer,
                  permission=MANAGE_CONTENT_PERMISSION)
class SharedContentDeleteForm(SharedContentWorkflowTransitionForm):
    """Shared content delete form"""

    @property
    def fields(self):
        fields = super().fields
        state = IWorkflowState(self.context)
        if state.version_id == 1:  # content deletion
            fields = fields.omit('comment')
        return fields

    def update_actions(self):
        super().update_actions()
        action = self.actions.get('action')
        if action is not None:
            state = IWorkflowState(self.context)
            if state.version_id == 1:  # remove the first and only version => remove all
                action.add_class('btn-danger')
                action.title = _("Delete definitively")

    def create_and_add(self, data):
        data = data.get(self, data)
        state = IWorkflowState(self.context)
        if state.version_id == 1:  # remove the first and only version => remove all
            content = get_parent(self.context, ISharedContent)
            parent = get_parent(content, IBaseSharedTool)
            del parent[content.__name__]
            target = self.request.registry.queryMultiAdapter((parent, self.request, self),
                                                             IWorkflowDeleteFormTarget)
        else:
            versions = IWorkflowVersions(self.context)
            versions.remove_version(state.version_id,
                                    state=DELETED, comment=data.get('comment'))
            target = versions.get_last_versions(count=1)[0]
        return target


@adapter_config(required=(IBaseSharedTool, IAdminLayer, SharedContentDeleteForm),
                provides=IWorkflowDeleteFormTarget)
def shared_content_workflow_delete_form_target(context, request, form):
    """Shared content workflow delete form target"""
    return context


@adapter_config(required=(IWorkflowVersion, IAdminLayer, SharedContentDeleteForm),
                provides=IAJAXFormRenderer)
class SharedContentDeleteFormRenderer(ContextRequestViewAdapter):
    """Shared content delete form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        return {
            'status': 'redirect',
            'location': absolute_url(changes, self.request, 'admin')
        }


@viewlet_config(name='help',
                context=IWorkflowVersion, layer=IAdminLayer, view=SharedContentDeleteForm,
                manager=IFormHeaderViewletManager, weight=10)
class SharedContentDeleteFormHelp(AlertMessage):
    """Shared content delete form help"""

    status = 'danger'

    @property
    def _message(self):
        state = IWorkflowState(self.context)
        if state.version_id == 1:
            return _("This content was never published and is going to be deleted.<br />"
                     "If you confirm deletion, it won't be possible to restore it.")
        return _("The content version is going to be definitely deleted.<br />"
                 "Will only remain the currently published or archived versions.")

    message_renderer = 'markdown'



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.httpexceptions import HTTPNotFound
from pyramid.renderers import render, render_to_response
from pyramid.response import Response
from pyramid.view import view_config
from zope.interface import Interface

from pyams_content.interfaces import MANAGE_REFERENCE_TABLE_PERMISSION
from pyams_content.reference.pictogram import IPictogram, IPictogramTable
from pyams_content.reference.pictogram.zmi.table import PictogramTableContainerTable
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.view import IModalEditForm, IModalPage
from pyams_skin.viewlet.actions import ContextAddAction
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces.form import NO_VALUE_STRING
from pyams_utils.interfaces.intids import IUniqueID
from pyams_utils.registry import get_utility
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.helper.event import get_json_table_row_add_callback, \
    get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IToolbarViewletManager
from pyams_zmi.table import TableElementEditor
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='add-pictogram.menu',
                context=IPictogramTable, layer=IAdminLayer, view=PictogramTableContainerTable,
                manager=IToolbarViewletManager, weight=10,
                permission=MANAGE_REFERENCE_TABLE_PERMISSION)
class PictogramAddAction(ContextAddAction):
    """Pictogram add action"""

    label = _("Add pictogram")
    href = 'add-pictogram.html'


@ajax_form_config(name='add-pictogram.html',
                  context=IPictogramTable, layer=IPyAMSLayer,
                  permission=MANAGE_REFERENCE_TABLE_PERMISSION)
class PictogramAddForm(AdminModalAddForm):
    """Pictogram add form"""

    subtitle = _("New pictogram")
    legend = _("New pictogram properties")
    modal_class = 'modal-xl'

    fields = Fields(IPictogram).omit('__parent__', '__name__')
    content_factory = IPictogram

    def add(self, obj):
        oid = IUniqueID(obj).oid
        self.context[oid] = obj


@adapter_config(required=(IPictogramTable, IAdminLayer, IModalPage),
                provides=IFormTitle)
def pictogram_add_form_title(context, request, view):
    """Pictogram add form title"""
    return get_object_label(context, request, view)


@adapter_config(required=(IPictogramTable, IAdminLayer, PictogramAddForm),
                provides=IAJAXFormRenderer)
class PictogramAddFormAJAXRenderer(ContextRequestViewAdapter):
    """Pictogram add form AJAX renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        table = get_parent(self.context, IPictogramTable)
        return {
            'callbacks': [
                get_json_table_row_add_callback(table, self.request,
                                                PictogramTableContainerTable, changes)
            ]
        }


@adapter_config(required=(IPictogram, IAdminLayer, Interface),
                provides=ITableElementEditor)
class PictogramElementEditor(TableElementEditor):
    """Pictogram table element editor"""


@ajax_form_config(name='properties.html',
                  context=IPictogram, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class PictogramEditForm(AdminModalEditForm):
    """Pictogram properties edit form"""

    @property
    def subtitle(self):
        translate = self.request.localizer.translate
        return translate(_("Pictogram: {}")).format(
            II18n(self.context).query_attribute('title', request=self.request))

    legend = _("Pictogram properties")
    modal_class = 'modal-xl'

    fields = Fields(IPictogram).omit('__parent__', '__name__')


@adapter_config(required=(IPictogram, IAdminLayer, IModalEditForm),
                provides=IFormTitle)
def pictogram_edit_form_title(context, request, form):
    """Pictogram edit form title"""
    table = get_utility(IPictogramTable)
    return TITLE_SPAN.format(get_object_label(table, request, form))


@adapter_config(required=(IPictogram, IAdminLayer, PictogramEditForm),
                provides=IAJAXFormRenderer)
class PictogramEditFormAJAXRenderer(ContextRequestViewAdapter):
    """Pictogram edit form AJAX renderer"""

    def render(self, changes):
        """AJAX result renderer"""
        if not changes:
            return None
        table = get_parent(self.context, IPictogramTable)
        return {
            'callbacks': [
                get_json_table_row_refresh_callback(table, self.request,
                                                    PictogramTableContainerTable, self.context)
            ]
        }


@view_config(name='get-pictogram-header.html',
             context=IPictogramTable, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION)
def get_pictogram_header_view(request):
    """View used to get thumbnail and alternate label associated with a given pictogram"""
    translate = request.localizer.translate
    name = request.params.get('value')
    if (not name) or (name == NO_VALUE_STRING):
        return Response(translate(_("Default header: --")))
    pictogram = request.context.get(name)
    if pictogram is None:
        raise HTTPNotFound()
    return render_to_response('templates/pictogram-header.pt', {
        'context': pictogram
    }, request=request)


def get_pictogram_header(pictogram, request=None):
    """Get thumbnail and alternate label associated with a given pictogram"""
    return render('templates/pictogram-header.pt', {
        'context': pictogram
    }, request=request)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyramid.view import view_config
from zope.interface import Interface, alsoProvides

from pyams_content.component.thesaurus import ICollectionsManager, ITagsManager, IThemesManager
from pyams_content.feature.filter import ICollectionsFilter, IFilter, ITagsFilter, IThemesFilter, ITitleFilter
from pyams_content.feature.filter.interfaces import IContentTypesFilter, IFiltersContainer, IThesaurusFilter
from pyams_content.zmi import content_js
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortletSettings, MANAGE_TEMPLATE_PERMISSION
from pyams_portal.zmi.interfaces import IPortletRendererSettingsEditForm
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_skin.interfaces.view import IModalPage
from pyams_skin.interfaces.viewlet import IContentSuffixViewletManager
from pyams_skin.viewlet.menu import MenuItem
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config, query_adapter
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.interfaces.data import IObjectData
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm, AdminModalEditForm
from pyams_zmi.helper.container import delete_container_element, switch_element_attribute
from pyams_zmi.helper.event import get_json_table_row_add_callback, get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.form import IFormTitle, IPropertiesEditForm
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager
from pyams_zmi.table import InnerTableAdminView, NameColumn, ReorderColumn, SortableTable, TableElementEditor, \
    TrashColumn, VisibilityColumn
from pyams_zmi.utils import get_object_label

__docformat__ = 'restructuredtext'

from pyams_content import _


class FiltersTable(SortableTable):
    """Filters table"""
    
    container_class = IFiltersContainer
    
    display_if_empty = True
    
    
@adapter_config(required=(IFiltersContainer, IAdminLayer, FiltersTable),
                provides=IValues)
class FiltersTableValues(ContextRequestViewAdapter):
    """Filters table values adapter"""
    
    @property
    def values(self):
        """Filter tables values getter"""
        yield from self.context.values()
        
        
@adapter_config(name='reorder',
                required=(IFiltersContainer, IAdminLayer, FiltersTable),
                provides=IColumn)
class FiltersTableReorderColumn(ReorderColumn):
    """Filters table reorder column"""


@view_config(name='reorder.json',
             context=IFiltersContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=MANAGE_TEMPLATE_PERMISSION)
def reorder_filters_table(request):
    """Reorder filters table"""
    order = request.params.get('order').split(';')
    request.context.updateOrder(order)
    return {
        'status': 'success',
        'closeForm': False
    }


@adapter_config(name='visible',
                required=(IFiltersContainer, IAdminLayer, FiltersTable),
                provides=IColumn)
class FiltersTableVisibleColumn(VisibilityColumn):
    """Filters table visible column"""

    hint = _("Click icon to show or hide filter")


@view_config(name='switch-visible-item.json',
             context=IFiltersContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True)
def switch_visible_filter(request):
    """Switch visible filter"""
    return switch_element_attribute(request)


@adapter_config(name='title',
                required=(IFiltersContainer, IAdminLayer, FiltersTable),
                provides=IColumn)
class FiltersTableTitleColumn(NameColumn):
    """Filters table name column"""

    i18n_header = _("Label")


@adapter_config(name='trash',
                required=(IFiltersContainer, IAdminLayer, FiltersTable),
                provides=IColumn)
class FiltersTableTrashColumn(TrashColumn):
    """Filters table trash column"""


@view_config(name='delete-element.json',
             context=IFiltersContainer, request_type=IPyAMSLayer,
             renderer='json', xhr=True,
             permission=MANAGE_TEMPLATE_PERMISSION)
def delete_filter(request):
    """Delete filter"""
    return delete_container_element(request)


@viewlet_config(name='filters-content-table',
                context=IFiltersContainer, layer=IAdminLayer,
                view=IPropertiesEditForm,
                manager=IContentSuffixViewletManager, weight=10)
@viewlet_config(name='filters-content-table',
                context=IFiltersContainer, layer=IAdminLayer,
                view=IPortletRendererSettingsEditForm,
                manager=IContentSuffixViewletManager, weight=10)
class FiltersTableView(InnerTableAdminView):
    """Filters table view"""

    table_class = FiltersTable
    table_label = _("List of filters")


#
# Filters forms
#

class FilterAddMenu(MenuItem):
    """Filter add menu"""

    modal_target = True

    def get_href(self):
        return absolute_url(self.context, self.request, self.href)


class FilterAddForm(AdminModalAddForm):
    """Base filter add form"""

    subtitle = _("New filter")
    legend = _("New filter properties")

    fields = Fields(IFilter).omit('visible')

    def add(self, obj):
        self.context.append(obj)


@adapter_config(required=(IFiltersContainer, IAdminLayer, FilterAddForm),
                provides=IFormTitle)
def filter_add_form_title(context, request, view):
    """Filter add form title"""
    target = get_parent(context, IPortletSettings)
    return query_adapter(IFormTitle, request, target, view)


@adapter_config(required=(IFiltersContainer, IAdminLayer, FilterAddForm),
                provides=IAJAXFormRenderer)
class FilterAddFormRenderer(ContextRequestViewAdapter):
    """Filter add form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        return {
            'status': 'success',
            'callbacks': [
                get_json_table_row_add_callback(self.context, self.request,
                                                FiltersTable, changes)
            ]
        }


@adapter_config(required=(IFilter, IAdminLayer, Interface),
                provides=ITableElementEditor)
class FilterElementEditor(TableElementEditor):
    """Filter element editor"""


@adapter_config(required=(IFilter, IAdminLayer, IModalPage),
                provides=IFormTitle)
def filter_edit_form_title(context, request, view):
    """Filter edit form title"""
    target = get_parent(context, IPortletSettings)
    return query_adapter(IFormTitle, request, target, view)


class FilterEditForm(AdminModalEditForm):
    """Filter base edit form"""

    @property
    def subtitle(self):
        """Form title getter"""
        translate = self.request.localizer.translate
        return translate(_("Filter: {}")).format(get_object_label(self.context, self.request, self))

    legend = _("Filter properties")
    fields = Fields(IFilter).omit('visible')


@adapter_config(required=(IFilter, IAdminLayer, FilterEditForm),
                provides=IAJAXFormRenderer)
class FilterEditFormRenderer(ContextRequestViewAdapter):
    """Filter edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        return {
            'status': 'success',
            'callbacks': [
                get_json_table_row_refresh_callback(self.context.__parent__, self.request,
                                                    FiltersTable, self.context)
            ]
        }


#
# Title filter forms
#

@viewlet_config(name='add-title-filter.menu',
                context=IFiltersContainer, layer=IAdminLayer, view=FiltersTable,
                manager=IContextAddingsViewletManager, weight=10,
                permission=MANAGE_TEMPLATE_PERMISSION)
class TitleFilterAddMenu(FilterAddMenu):
    """Title filter add menu"""

    label = _("Title filter")
    href = 'add-title-filter.html'


@ajax_form_config(name='add-title-filter.html',
                  context=IFiltersContainer, layer=IPyAMSLayer,
                  permission=MANAGE_TEMPLATE_PERMISSION)
class TitleFilterAddForm(FilterAddForm):
    """Title filter add form"""

    fields = Fields(ITitleFilter).omit('visible')
    content_factory = ITitleFilter


@ajax_form_config(name='properties.html',
                  context=ITitleFilter, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class TitleFilterEditForm(FilterEditForm):
    """Title filter edit form"""

    fields = Fields(ITitleFilter).omit('visible')


#
# Content-type filter forms
#

@viewlet_config(name='add-content-type-filter.menu',
                context=IFiltersContainer, layer=IAdminLayer, view=FiltersTable,
                manager=IContextAddingsViewletManager, weight=15,
                permission=MANAGE_TEMPLATE_PERMISSION)
class ContentTypeFilterAddMenu(FilterAddMenu):
    """Content-type filter add menu"""

    label = _("Content-type filter")
    href = 'add-content-type-filter.html'


@ajax_form_config(name='add-content-type-filter.html',
                  context=IFiltersContainer, layer=IPyAMSLayer,
                  permission=MANAGE_TEMPLATE_PERMISSION)
class ContentTypeFilterAddForm(FilterAddForm):
    """Content-type filter add form"""

    fields = Fields(IContentTypesFilter).omit('visible')
    content_factory = IContentTypesFilter


@ajax_form_config(name='properties.html',
                  context=IContentTypesFilter, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ContentTypeFilterEditForm(FilterEditForm):
    """Content-type filter edit form"""

    fields = Fields(IContentTypesFilter).omit('visible')


#
# Thesaurus-based filters forms
#

class ThesaurusFilterAddForm(FilterAddForm):
    """Thesaurus-based filter add form"""

    fields = Fields(IThesaurusFilter).omit('visible')

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        thesaurus_name = self.widgets.get('thesaurus_name')
        if thesaurus_name is not None:
            thesaurus_name.prompt = True
            thesaurus_name.prompt_message = _("(no selected thesaurus)")
            thesaurus_name.object_data = {
                'ams-modules': {
                    'content': {
                        'src': get_resource_path(content_js)
                    }
                },
                'ams-change-handler': 'MyAMS.content.thesaurus.changeThesaurus'
            }
            alsoProvides(thesaurus_name, IObjectData)


@ajax_form_config(name='properties.html',
                  context=IThesaurusFilter, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ThesaurusFilterEditForm(FilterEditForm):
    """Thesaurus-based filter edit form"""

    fields = Fields(IThesaurusFilter).omit('visible')

    def update_widgets(self, prefix=None):
        # store thesaurus name in request header to be able to set
        # extract name correctly
        if self.request.method == 'POST':
            name = f'{self.prefix}widgets.thesaurus_name'
            value = self.request.params.get(name)
            if value is not None:
                self.request.headers['X-Thesaurus-Name'] = value
        super().update_widgets(prefix)
        thesaurus_name = self.widgets.get('thesaurus_name')
        if thesaurus_name is not None:
            thesaurus_name.prompt = True
            thesaurus_name.prompt_message = _("(no selected thesaurus)")
            thesaurus_name.object_data = {
                'ams-modules': {
                    'content': {
                        'src': get_resource_path(content_js)
                    }
                },
                'ams-change-handler': 'MyAMS.content.thesaurus.changeThesaurus'
            }
            alsoProvides(thesaurus_name, IObjectData)


@viewlet_config(name='add-tag-filter.menu',
                context=IFiltersContainer, layer=IAdminLayer, view=FiltersTable,
                manager=IContextAddingsViewletManager, weight=20,
                permission=MANAGE_TEMPLATE_PERMISSION)
class TagsFilterAddMenu(FilterAddMenu):
    """Tags filter add menu"""

    def __new__(cls, context, request, view, manager):
        tags_manager = ITagsManager(request.root, None)
        if (tags_manager is None) or not tags_manager.thesaurus_name:
            return None
        return FilterAddMenu.__new__(cls)

    label = _("Tag filter")
    href = 'add-tag-filter.html'


@ajax_form_config(name='add-tag-filter.html',
                  context=IFiltersContainer, layer=IPyAMSLayer,
                  permission=MANAGE_TEMPLATE_PERMISSION)
class TagsFilterAddForm(ThesaurusFilterAddForm):
    """Tags filter add form"""

    content_factory = ITagsFilter


@viewlet_config(name='add-theme-filter.menu',
                context=IFiltersContainer, layer=IAdminLayer, view=FiltersTable,
                manager=IContextAddingsViewletManager, weight=30,
                permission=MANAGE_TEMPLATE_PERMISSION)
class ThemesFilterAddMenu(FilterAddMenu):
    """Themes filter add menu"""

    def __new__(cls, context, request, view, manager):
        themes_manager = IThemesManager(request.root, None)
        if (themes_manager is None) or not themes_manager.thesaurus_name:
            return None
        return FilterAddMenu.__new__(cls)

    label = _("Theme filter")
    href = 'add-theme-filter.html'


@ajax_form_config(name='add-theme-filter.html',
                  context=IFiltersContainer, layer=IPyAMSLayer,
                  permission=MANAGE_TEMPLATE_PERMISSION)
class ThemesFilterAddForm(ThesaurusFilterAddForm):
    """Themes filter add form"""

    content_factory = IThemesFilter


@viewlet_config(name='add-collection-filter.menu',
                context=IFiltersContainer, layer=IAdminLayer, view=FiltersTable,
                manager=IContextAddingsViewletManager, weight=40,
                permission=MANAGE_TEMPLATE_PERMISSION)
class CollectionsFilterAddMenu(FilterAddMenu):
    """Collections filter add menu"""

    def __new__(cls, context, request, view, manager):
        collections_manager = ICollectionsManager(request.root, None)
        if (collections_manager is None) or not collections_manager.thesaurus_name:
            return None
        return FilterAddMenu.__new__(cls)

    label = _("Collection filter")
    href = 'add-collection-filter.html'


@ajax_form_config(name='add-collection-filter.html',
                  context=IFiltersContainer, layer=IPyAMSLayer,
                  permission=MANAGE_TEMPLATE_PERMISSION)
class CollectionsFilterAddForm(ThesaurusFilterAddForm):
    """Collections filter add form"""

    content_factory = ICollectionsFilter
